@@ -48,11 +48,39 @@
 #include <sc_containers.h>
 #define _p4est_const _sc_const
 
+/*--------------------------------------------------------------------*/
+/*------------------------ QUERY API CHANGES -------------------------*/
+/*---- definitions to allow user code to query the p4est library -----*/
+
+/** We do no longer dereference unneeded pointers in p4est_transfer_. */
+#define P4EST_COMM_TRANSFER_NULL
+
+/** The \ref p4est_connectivity_new_disk function now accepts a bool arg.
+ * The same holds for \ref p4est_wrap_new_disk. */
+#define P4EST_CONN_DISK_PERIODIC
+
+/** The \ref p4est_connectivity_reorder_newid function exists. */
+#define P4EST_CONN_REORDER_NEWID
+
+/** The \ref p4est_search_local function replaces \ref p4est_search.
+ * The latter function is still available with updated internal semantics.
+ * Furthermore, we have added \ref p4est_search_partition to search
+ * the parallel partition without accessing any local elements,
+ * and \ref p4est_search_all that combines the two.
+ */
+#define P4EST_SEARCH_LOCAL
+
+/** We expose the \ref p4est_vtk_write_cell_datav function. */
+#define P4EST_VTK_CELL_DATAV
+
+/*--------------------------------------------------------------------*/
+
 SC_EXTERN_C_BEGIN;
 
 /** Typedef for quadrant coordinates. */
 typedef int32_t     p4est_qcoord_t;
 #define p4est_qcoord_compare sc_int32_compare
+#define P4EST_QCOORD_BITS 32
 #define P4EST_MPI_QCOORD sc_MPI_INT
 #define P4EST_VTK_QCOORD "Int32"
 #define P4EST_F90_QCOORD INTEGER(KIND=C_INT32_T)
@@ -64,6 +92,7 @@ typedef int32_t     p4est_qcoord_t;
 /** Typedef for counting topological entities (trees, tree vertices). */
 typedef int32_t     p4est_topidx_t;
 #define p4est_topidx_compare sc_int32_compare
+#define P4EST_TOPIDX_BITS 32
 #define P4EST_MPI_TOPIDX sc_MPI_INT
 #define P4EST_VTK_TOPIDX "Int32"
 #define P4EST_F90_TOPIDX INTEGER(KIND=C_INT32_T)
@@ -76,6 +105,7 @@ typedef int32_t     p4est_topidx_t;
 /** Typedef for processor-local indexing of quadrants and nodes. */
 typedef int32_t     p4est_locidx_t;
 #define p4est_locidx_compare sc_int32_compare
+#define P4EST_LOCIDX_BITS 32
 #define P4EST_MPI_LOCIDX sc_MPI_INT
 #define P4EST_VTK_LOCIDX "Int32"
 #define P4EST_F90_LOCIDX INTEGER(KIND=C_INT32_T)
@@ -87,6 +117,7 @@ typedef int32_t     p4est_locidx_t;
 /** Typedef for globally unique indexing of quadrants. */
 typedef int64_t     p4est_gloidx_t;
 #define p4est_gloidx_compare sc_int64_compare
+#define P4EST_GLOIDX_BITS 64
 #define P4EST_MPI_GLOIDX sc_MPI_LONG_LONG_INT
 #define P4EST_VTK_GLOIDX "Int64"
 #define P4EST_F90_GLOIDX INTEGER(KIND=C_INT64_T)
@@ -138,6 +169,16 @@ p4est_comm_tag_t;
   do { int _p4est_i = (int) (expression);                               \
        SC_CHECK_ABORT (_p4est_i, "Expected true: '" #expression "'");   \
   } while (0)
+#define P4EST_EXECUTE_ASSERT_INT(expression,ival)                       \
+  do { int _p4est_i = (int) (expression);                               \
+       SC_CHECK_ABORT ((ival) == _p4est_i,                              \
+                       "Expected '" #ival "': '" #expression "'");      \
+  } while (0)
+#define P4EST_EXECUTE_ASSERT_TOPIDX(expression,tval)                    \
+  do { p4est_topidx_t _p4est_t = (p4est_topidx_t) (expression);         \
+       SC_CHECK_ABORT ((tval) == _p4est_t,                              \
+                       "Expected '" #tval "': '" #expression "'");      \
+  } while (0)
 #define P4EST_DEBUG_EXECUTE(expression)                 \
   do { (void) (expression); } while (0)
 #else
@@ -146,6 +187,10 @@ p4est_comm_tag_t;
   do { (void) (expression); } while (0)
 #define P4EST_EXECUTE_ASSERT_TRUE(expression)           \
   do { (void) (expression); } while (0)
+#define P4EST_EXECUTE_ASSERT_INT(expression,ival)       \
+  do { (void) (expression); } while (0)
+#define P4EST_EXECUTE_ASSERT_TOPIDX(expression,tval)    \
+  do { (void) (expression); } while (0)
 #define P4EST_DEBUG_EXECUTE(expression) SC_NOOP ()
 #endif
 
@@ -277,13 +322,13 @@ void                P4EST_LERRORF (const char *fmt, ...)
 extern int          p4est_package_id;
 
 static inline void
-p4est_log_indent_push ()
+p4est_log_indent_push (void)
 {
   sc_log_indent_push_count (p4est_package_id, 1);
 }
 
 static inline void
-p4est_log_indent_pop ()
+p4est_log_indent_pop (void)
 {
   sc_log_indent_pop_count (p4est_package_id, 1);
 }
@@ -300,6 +345,30 @@ p4est_log_indent_pop ()
 void                p4est_init (sc_log_handler_t log_handler,
                                 int log_threshold);
 
+/** Return whether p4est has been initialized or not.
+ * Keep in mind that \ref p4est_init is an optional function
+ * but it helps with proper parallel logging.
+ *
+ * Currently there is no inverse to \ref p4est_init, and no way to deinit it.
+ * This is ok since initialization generally does no harm.
+ * Just do not call libsc's finalize function while p4est is still in use.
+ *
+ * \return          True if p4est has been initialized with a call to
+ *                  \ref p4est_init and false otherwise.
+ */
+int                 p4est_is_initialized (void);
+
+/** Check for a sufficiently recent zlib installation.
+ * \return          True if zlib is detected in both sc and p4est.
+ */
+int                 p4est_have_zlib (void);
+
+/** Query the package identity as registered in libsc.
+ * \return          This is -1 before \ref p4est_init has been called
+ *                  and a proper package identifier (>= 0) afterwards.
+ */
+int                 p4est_get_package_id (void);
+
 /** Compute hash value for two p4est_topidx_t integers.
  * \param [in] tt     Array of (at least) two values.
  * \return            An unsigned hash value.
@@ -477,6 +546,28 @@ p4est_partition_cut_gloidx (p4est_gloidx_t global_num, int p, int num_procs)
   return result;
 }
 
+/** Return the full version of p4est.
+ *
+ * \return          Return the version of p4est using the format
+ *                  `VERSION_MAJOR.VERSION_MINOR.VERSION_POINT`,
+ *                  where `VERSION_POINT` can contain dots and
+ *                  characters, e.g. to indicate the additional
+ *                  number of commits and a git commit hash.
+ */
+const char         *p4est_version (void);
+
+/** Return the major version of p4est.
+ *
+ * \return          Return the major version of p4est.
+ */
+int                 p4est_version_major (void);
+
+/** Return the minor version of p4est.
+ *
+ * \return          Return the minor version of p4est.
+ */
+int                 p4est_version_minor (void);
+
 SC_EXTERN_C_END;
 
 #endif /* !P4EST_BASE_H */
