@@ -24,10 +24,274 @@
 
 #ifdef P4_TO_P8
 #include <p8est_bits.h>
+#include <p8est_extended.h>
 #else
 #include <p4est_bits.h>
+#include <p4est_extended.h>
 #endif /* !P4_TO_P8 */
 
+/* Function declarations for 128 bit unsigned integers
+ * are in p{4,8}est_extended.h. */
+int
+p4est_lid_compare (const p4est_lid_t * a, const p4est_lid_t * b)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  return sc_uint128_compare (a, b);
+#else
+  if (*a < *b)
+    return -1;
+  else if (*b < *a)
+    return 1;
+  return 0;
+#endif
+}
+
+int
+p4est_lid_is_equal (const p4est_lid_t * a, const p4est_lid_t * b)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  return sc_uint128_is_equal (a, b);
+#else
+  return *a == *b;
+#endif
+}
+
+void
+p4est_lid_init (p4est_lid_t * input, uint64_t high, uint64_t low)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_init (input, high, low);
+#else
+  SC_ASSERT (high == 0);
+  *input = (p4est_lid_t) low;
+#endif
+}
+
+void
+p4est_lid_set_zero (p4est_lid_t * input)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_init (input, 0, 0);
+#else
+  *input = (p4est_lid_t) 0;
+#endif
+}
+
+void
+p4est_lid_set_one (p4est_lid_t * input)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_init (input, 0, 1);
+#else
+  *input = (p4est_lid_t) 1;
+#endif
+}
+
+void
+p4est_lid_set_uint64 (p4est_lid_t * input, uint64_t u)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_init (input, 0, u);
+#else
+  *input = u;
+#endif
+}
+
+int
+p4est_lid_chk_bit (const p4est_lid_t * input, int bit_number)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  return sc_uint128_chk_bit (input, bit_number);
+#else
+  P4EST_ASSERT (bit_number >= 0 && bit_number < 64);
+  return (*input & ((uint64_t) 1) << bit_number) != 0;
+#endif
+}
+
+void
+p4est_lid_set_bit (p4est_lid_t * input, int bit_number)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_set_bit (input, bit_number);
+#else
+  P4EST_ASSERT (bit_number >= 0 && bit_number < 64);
+  *input |= ((uint64_t) 1) << bit_number;
+#endif
+}
+
+void
+p4est_lid_copy (const p4est_lid_t * input, p4est_lid_t * output)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_copy (input, output);
+  P4EST_ASSERT (output != NULL);
+#else
+  *output = *input;
+#endif
+}
+
+void
+p4est_lid_add (const p4est_lid_t * a, const p4est_lid_t * b,
+               p4est_lid_t * result)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_add (a, b, result);
+  P4EST_ASSERT (result != NULL);
+#else
+  *result = *a + *b;
+#endif
+}
+
+void
+p4est_lid_sub (const p4est_lid_t * a, const p4est_lid_t * b,
+               p4est_lid_t * result)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_sub (a, b, result);
+  P4EST_ASSERT (result != NULL);
+#else
+  *result = *a - *b;
+#endif
+}
+
+void
+p4est_lid_bitwise_neg (const p4est_lid_t * a, p4est_lid_t * result)
+{
+  P4EST_ASSERT (a != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_bitwise_neg (a, result);
+  P4EST_ASSERT (result != NULL);
+#else
+  *result = ~(*a);
+#endif
+}
+
+void
+p4est_lid_bitwise_or (const p4est_lid_t * a, const p4est_lid_t * b,
+                      p4est_lid_t * result)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_bitwise_or (a, b, result);
+  P4EST_ASSERT (result != NULL);
+#else
+  *result = *a | *b;
+#endif
+}
+
+void
+p4est_lid_bitwise_and (const p4est_lid_t * a, const p4est_lid_t * b,
+                       p4est_lid_t * result)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_bitwise_and (a, b, result);
+  P4EST_ASSERT (result != NULL);
+#else
+  *result = *a & *b;
+#endif
+}
+
+void
+p4est_lid_shift_right (const p4est_lid_t * input, unsigned shift_count,
+                       p4est_lid_t * result)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_shift_right (input, shift_count, result);
+  P4EST_ASSERT (result != NULL);
+#else
+  *result = *input >> shift_count;
+#endif
+}
+
+void
+p4est_lid_shift_left (const p4est_lid_t * input, unsigned shift_count,
+                      p4est_lid_t * result)
+{
+  P4EST_ASSERT (input != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_shift_left (input, shift_count, result);
+  P4EST_ASSERT (result != NULL);
+#else
+  *result = *input << shift_count;
+#endif
+}
+
+void
+p4est_lid_add_inplace (p4est_lid_t * a, const p4est_lid_t * b)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_add_inplace (a, b);
+#else
+  *a += *b;
+#endif
+}
+
+void
+p4est_lid_sub_inplace (p4est_lid_t * a, const p4est_lid_t * b)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_sub_inplace (a, b);
+#else
+  *a -= *b;
+#endif
+}
+
+void
+p4est_lid_bitwise_or_inplace (p4est_lid_t * a, const p4est_lid_t * b)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_bitwise_or_inplace (a, b);
+#else
+  *a |= *b;
+#endif
+}
+
+void
+p4est_lid_bitwise_and_inplace (p4est_lid_t * a, const p4est_lid_t * b)
+{
+  P4EST_ASSERT (a != NULL && b != NULL);
+#ifdef P4_TO_P8
+  sc_uint128_bitwise_and_inplace (a, b);
+#else
+  *a &= *b;
+#endif
+}
+
+static void
+p4est_qcoord_set_bit (p4est_qcoord_t * input, int bit_number)
+{
+  P4EST_ASSERT (input != NULL);
+  P4EST_ASSERT (bit_number >= 0 && bit_number < P4EST_QCOORD_BITS);
+  *input |= ((p4est_qcoord_t) 1) << bit_number;
+}
+
+void
+p4est_quadrant_pad (p4est_quadrant_t * q)
+{
+  P4EST_ASSERT (q != NULL);
+
+  /* *INDENT-OFF* HORRIBLE indent bug */
+  q->pad8 = (int8_t) -1;
+  q->pad16 = (int16_t) -1;
+  /* *INDENT-ON* */
+}
+
 void
 p4est_quadrant_print (int log_priority, const p4est_quadrant_t * q)
 {
@@ -55,13 +319,25 @@ p4est_quadrant_is_equal (const p4est_quadrant_t * q1,
     ;
 }
 
+void
+p4est_quadrant_copy (const p4est_quadrant_t * q, p4est_quadrant_t * copy)
+{
+  P4EST_ASSERT (p4est_quadrant_is_extended (q));
+
+  copy->x = q->x;
+  copy->y = q->y;
+#ifdef P4_TO_P8
+  copy->z = q->z;
+#endif
+  copy->level = q->level;
+}
+
 int
 p4est_quadrant_overlaps (const p4est_quadrant_t * q1,
                          const p4est_quadrant_t * q2)
 {
   int8_t              level = SC_MIN (q1->level, q2->level);
-  p4est_qcoord_t      mask =
-    ((p4est_qcoord_t) - 1) << (P4EST_MAXLEVEL - level);
+  p4est_qcoord_t      mask = ~((1 << (P4EST_MAXLEVEL - level)) - 1);
 
   if (((q1->x ^ q2->x) & mask) || ((q1->y ^ q2->y) & mask)
 #ifdef P4_TO_P8
@@ -83,41 +359,34 @@ p4est_quadrant_is_equal_piggy (const p4est_quadrant_t * q1,
 }
 
 int
-p4est_quadrant_compare (const void *v1, const void *v2)
+p4est_coordinates_compare (const p4est_qcoord_t v1[],
+                           const p4est_qcoord_t v2[])
 {
-  const p4est_quadrant_t *q1 = (const p4est_quadrant_t *) v1;
-  const p4est_quadrant_t *q2 = (const p4est_quadrant_t *) v2;
-
   uint32_t            exclorx, exclory, exclorxy, exclor;
 #ifdef P4_TO_P8
   uint32_t            exclorz;
 #endif
   int64_t             p1, p2, diff;
 
-  P4EST_ASSERT (p4est_quadrant_is_node (q1, 1) ||
-                p4est_quadrant_is_extended (q1));
-  P4EST_ASSERT (p4est_quadrant_is_node (q2, 1) ||
-                p4est_quadrant_is_extended (q2));
-
   /* these are unsigned variables that inherit the sign bits */
-  exclorx = q1->x ^ q2->x;
-  exclory = q1->y ^ q2->y;
+  exclorx = v1[0] ^ v2[0];
+  exclory = v1[1] ^ v2[1];
   exclor = exclorxy = exclorx | exclory;
 #ifdef P4_TO_P8
-  exclorz = q1->z ^ q2->z;
+  exclorz = v1[2] ^ v2[2];
   exclor = exclorxy | exclorz;
 #endif
 
   if (!exclor) {
-    return (int) q1->level - (int) q2->level;
+    return 0;
   }
 
 #ifdef P4_TO_P8
   /* if (exclor ^ exclorz) > exclorz, then exclorxy has a more significant bit
    * than exclorz; also exclor and (exclor ^ exclorz) cannot be equal */
   if (exclorz > (exclor ^ exclorz)) {
-    p1 = q1->z + ((q1->z >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
-    p2 = q2->z + ((q2->z >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
+    p1 = v1[2] + ((v1[2] >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
+    p2 = v2[2] + ((v2[2] >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
   }
   else
 #if 0
@@ -125,25 +394,52 @@ p4est_quadrant_compare (const void *v1, const void *v2)
 #endif
 #endif
   if (exclory > (exclorxy ^ exclory)) {
-    p1 = q1->y + ((q1->y >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
-    p2 = q2->y + ((q2->y >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
+    p1 = v1[1] + ((v1[1] >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
+    p2 = v2[1] + ((v2[1] >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
   }
   else {
-    p1 = q1->x + ((q1->x >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
-    p2 = q2->x + ((q2->x >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
+    p1 = v1[0] + ((v1[0] >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
+    p2 = v2[0] + ((v2[0] >= 0) ? 0 : ((int64_t) 1 << (P4EST_MAXLEVEL + 2)));
   }
   diff = p1 - p2;
   return (diff == 0) ? 0 : ((diff < 0) ? -1 : 1);
 }
 
+int
+p4est_quadrant_compare (const void *v1, const void *v2)
+{
+  const p4est_quadrant_t *q1 = (const p4est_quadrant_t *) v1;
+  const p4est_quadrant_t *q2 = (const p4est_quadrant_t *) v2;
+
+  p4est_qcoord_t      a[P4EST_DIM], b[P4EST_DIM];
+  int                 coord_diff;
+
+  P4EST_ASSERT (p4est_quadrant_is_node (q1, 1) ||
+                p4est_quadrant_is_extended (q1));
+  P4EST_ASSERT (p4est_quadrant_is_node (q2, 1) ||
+                p4est_quadrant_is_extended (q2));
+
+  a[0] = q1->x;
+  a[1] = q1->y;
+#ifdef P4_TO_P8
+  a[2] = q1->z;
+#endif
+  b[0] = q2->x;
+  b[1] = q2->y;
+#ifdef P4_TO_P8
+  b[2] = q2->z;
+#endif
+  coord_diff = p4est_coordinates_compare (a, b);
+  return coord_diff ? coord_diff : ((int) q1->level - (int) q2->level);
+}
+
 int
 p4est_quadrant_disjoint (const void *a, const void *b)
 {
   const p4est_quadrant_t *q = (p4est_quadrant_t *) a;
   const p4est_quadrant_t *r = (p4est_quadrant_t *) b;
   int8_t              level = SC_MIN (q->level, r->level);
-  p4est_qcoord_t      mask =
-    ((p4est_qcoord_t) - 1) << (P4EST_MAXLEVEL - level);
+  p4est_qcoord_t      mask = ~((1 << (P4EST_MAXLEVEL - level)) - 1);
 
   if (((q->x ^ r->x) & mask) || ((q->y ^ r->y) & mask)
 #ifdef P4_TO_P8
@@ -360,16 +656,31 @@ p4est_quadrant_child_id (const p4est_quadrant_t * q)
   return p4est_quadrant_ancestor_id (q, (int) q->level);
 }
 
+int
+p4est_coordinates_is_inside_root (const p4est_qcoord_t coord[])
+{
+  /* *INDENT-OFF* */
+  return (coord[0] >= 0 && coord[0] < P4EST_ROOT_LEN) &&
+         (coord[1] >= 0 && coord[1] < P4EST_ROOT_LEN) &&
+#ifdef P4_TO_P8
+         (coord[2] >= 0 && coord[2] < P4EST_ROOT_LEN) &&
+#endif
+  /* *INDENT-ON* */
+  1;
+}
+
 int
 p4est_quadrant_is_inside_root (const p4est_quadrant_t * q)
 {
-  return
-    (q->x >= 0 && q->x < P4EST_ROOT_LEN) &&
-    (q->y >= 0 && q->y < P4EST_ROOT_LEN) &&
+  p4est_qcoord_t      coord[P4EST_DIM];
+
+  coord[0] = q->x;
+  coord[1] = q->y;
 #ifdef P4_TO_P8
-    (q->z >= 0 && q->z < P4EST_ROOT_LEN) &&
+  coord[2] = q->z;
 #endif
-    1;
+
+  return p4est_coordinates_is_inside_root (coord);
 }
 
 int
@@ -439,16 +750,30 @@ p4est_quadrant_is_node (const p4est_quadrant_t * q, int inside)
 }
 
 int
-p4est_quadrant_is_valid (const p4est_quadrant_t * q)
+p4est_coordinates_is_valid (const p4est_qcoord_t coord[], int level)
 {
   return
-    (q->level >= 0 && q->level <= P4EST_QMAXLEVEL) &&
-    ((q->x & (P4EST_QUADRANT_LEN (q->level) - 1)) == 0) &&
-    ((q->y & (P4EST_QUADRANT_LEN (q->level) - 1)) == 0) &&
+    (level >= 0 && level <= P4EST_QMAXLEVEL) &&
+    ((coord[0] & (P4EST_QUADRANT_LEN (level) - 1)) == 0) &&
+    ((coord[1] & (P4EST_QUADRANT_LEN (level) - 1)) == 0) &&
 #ifdef P4_TO_P8
-    ((q->z & (P4EST_QUADRANT_LEN (q->level) - 1)) == 0) &&
+    ((coord[2] & (P4EST_QUADRANT_LEN (level) - 1)) == 0) &&
 #endif
-    p4est_quadrant_is_inside_root (q);
+    p4est_coordinates_is_inside_root (coord);
+}
+
+int
+p4est_quadrant_is_valid (const p4est_quadrant_t * q)
+{
+  p4est_qcoord_t      coord[P4EST_DIM];
+
+  coord[0] = q->x;
+  coord[1] = q->y;
+#ifdef P4_TO_P8
+  coord[2] = q->z;
+#endif
+
+  return p4est_coordinates_is_valid (coord, q->level);
 }
 
 int
@@ -661,7 +986,7 @@ p4est_quadrant_is_next (const p4est_quadrant_t * q,
                         const p4est_quadrant_t * r)
 {
   int                 minlevel;
-  uint64_t            i1, i2;
+  p4est_lid_t         i1, i2, one;
   p4est_qcoord_t      mask;
 
   P4EST_ASSERT (p4est_quadrant_is_extended (q));
@@ -684,17 +1009,19 @@ p4est_quadrant_is_next (const p4est_quadrant_t * q,
   else {
     minlevel = (int) q->level;
   }
-  i1 = p4est_quadrant_linear_id (q, minlevel);
-  i2 = p4est_quadrant_linear_id (r, minlevel);
+  p4est_quadrant_linear_id_ext128 (q, minlevel, &i1);
+  p4est_quadrant_linear_id_ext128 (r, minlevel, &i2);
 
-  return (i1 + 1 == i2);
+  p4est_lid_set_one (&one);
+  p4est_lid_add_inplace (&i1, &one);
+  return p4est_lid_is_equal (&i1, &i2);
 }
 
 int
 p4est_quadrant_is_next_D (const p4est_quadrant_t * q,
                           const p4est_quadrant_t * r)
 {
-  uint64_t            i1, i2;
+  p4est_lid_t         i1, i2, one;
   p4est_quadrant_t    a, b;
 
   /* validity of q and r is asserted in p4est_quadrant_compare */
@@ -710,10 +1037,12 @@ p4est_quadrant_is_next_D (const p4est_quadrant_t * q,
     }
     p4est_quadrant_parent (&a, &a);
   }
-  i1 = p4est_quadrant_linear_id (&a, (int) a.level);
-  i2 = p4est_quadrant_linear_id (&b, (int) a.level);
+  p4est_quadrant_linear_id_ext128 (&a, (int) a.level, &i1);
+  p4est_quadrant_linear_id_ext128 (&b, (int) a.level, &i2);
 
-  return (i1 + 1 == i2);
+  p4est_lid_set_one (&one);
+  p4est_lid_add_inplace (&i1, &one);
+  return p4est_lid_is_equal (&i1, &i2);
 }
 
 int
@@ -772,6 +1101,120 @@ p4est_quadrant_is_inside_tree (p4est_tree_t * tree,
   return 1;
 }
 
+int
+p4est_quadrant_is_first_last (const p4est_quadrant_t * f,
+                              const p4est_quadrant_t * l,
+                              const p4est_quadrant_t * a)
+{
+  p4est_qcoord_t      w;
+
+  P4EST_ASSERT (p4est_quadrant_compare (f, l) <= 0);
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, f) ||
+                p4est_quadrant_is_ancestor (a, f));
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, l) ||
+                p4est_quadrant_is_ancestor (a, l));
+
+  /* is the first corner a match? */
+  if (a->x != f->x || a->y != f->y
+#ifdef P4_TO_P8
+      || a->z != f->z
+#endif
+    ) {
+    return 0;
+  }
+
+  /* is the last corner a match? */
+  w = P4EST_QUADRANT_LEN (a->level) - P4EST_QUADRANT_LEN (l->level);
+  P4EST_ASSERT (w >= 0);
+  if (a->x + w != l->x || a->y + w != l->y
+#ifdef P4_TO_P8
+      || a->z + w != l->z
+#endif
+    ) {
+    return 0;
+  }
+
+  /* yes, they both are matches */
+  return 1;
+}
+
+void
+p4est_quadrant_enlarge_first (const p4est_quadrant_t * a,
+                              p4est_quadrant_t * q)
+{
+  p4est_qcoord_t      w;
+#ifdef P4EST_ENABLE_DEBUG
+  p4est_quadrant_t    inp = *q;
+#endif
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+
+  w = q->x | q->y
+#ifdef P4_TO_P8
+    | q->z
+#endif
+    ;
+  while (a->level < q->level && (w & P4EST_QUADRANT_LEN (q->level)) == 0) {
+    --q->level;
+  }
+  P4EST_ASSERT (p4est_quadrant_is_extended (q));
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+  P4EST_ASSERT (p4est_quadrant_is_equal (q, &inp) ||
+                p4est_quadrant_is_ancestor (q, &inp));
+
+  /* verify that the first corner stayed the same */
+  P4EST_ASSERT (q->x == inp.x && q->y == inp.y
+                P4EST_ONLY_P8_LAND (q->z == inp.z));
+}
+
+void
+p4est_quadrant_enlarge_last (const p4est_quadrant_t * a, p4est_quadrant_t * q)
+{
+  int                 inlevel;
+  p4est_qcoord_t      w;
+#ifdef P4EST_ENABLE_DEBUG
+  p4est_quadrant_t    inp = *q;
+#endif
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+
+  inlevel = (int) q->level;
+  w = q->x & q->y
+#ifdef P4_TO_P8
+    & q->z
+#endif
+    ;
+  while (a->level < q->level && (w & P4EST_QUADRANT_LEN (q->level)) != 0) {
+    --q->level;
+  }
+
+  /* move first corner to the lower left according to q's enlargement */
+  w = ~(P4EST_QUADRANT_LEN (q->level) - P4EST_QUADRANT_LEN (inlevel));
+  q->x &= w;
+  q->y &= w;
+#ifdef P4_TO_P8
+  q->z &= w;
+#endif
+  P4EST_ASSERT (p4est_quadrant_is_extended (q));
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+  P4EST_ASSERT (p4est_quadrant_is_equal (q, &inp) ||
+                p4est_quadrant_is_ancestor (q, &inp));
+
+  /* verify that the last corner stayed the same */
+#ifdef P4EST_ENABLE_DEBUG
+  P4EST_ASSERT (inlevel == inp.level);
+  P4EST_ASSERT (~w >= 0);
+  P4EST_ASSERT (q->x + ~w == inp.x && q->y + ~w == inp.y
+                P4EST_ONLY_P8_LAND (q->z + ~w == inp.z));
+#endif
+}
+
 void
 p4est_quadrant_ancestor (const p4est_quadrant_t * q,
                          int level, p4est_quadrant_t * r)
@@ -1058,8 +1501,7 @@ p4est_quadrant_corner_neighbor_extra (const p4est_quadrant_t * q,
 
   p4est_quadrant_corner_neighbor (q, corner, &temp);
   if (p4est_quadrant_is_inside_root (&temp)) {
-    qp = p4est_quadrant_array_push (quads);
-    *qp = temp;
+    qp = p4est_quadrant_array_push_copy (quads, &temp);
     tp = (p4est_topidx_t *) sc_array_push (treeids);
     *tp = t;
     if (ncorners != NULL) {
@@ -1071,7 +1513,7 @@ p4est_quadrant_corner_neighbor_extra (const p4est_quadrant_t * q,
 
   if (!p4est_quadrant_is_outside_corner (&temp)) {
 #ifndef P4_TO_P8
-    qp = (p4est_quadrant_t *) sc_array_push (quads);
+    qp = p4est_quadrant_array_push (quads);
     tp = (p4est_topidx_t *) sc_array_push (treeids);
 
     face = p4est_corner_faces[corner][0];
@@ -1441,6 +1883,73 @@ p4est_nearest_common_ancestor_D (const p4est_quadrant_t * q1,
   P4EST_ASSERT (p4est_quadrant_is_extended (r));
 }
 
+void
+p4est_coordinates_transform_face (const p4est_qcoord_t coords_in[],
+                                  p4est_qcoord_t coords_out[],
+                                  const int ftransform[])
+{
+  p4est_qcoord_t     *target_xyz[P4EST_DIM];
+  const p4est_qcoord_t *my_xyz[P4EST_DIM];
+  const int          *my_axis = &ftransform[0];
+  const int          *target_axis = &ftransform[3];
+  const int          *edge_reverse = &ftransform[6];
+
+#ifdef P4EST_ENABLE_DEBUG
+  int                 i;
+
+  for (i = 0; i < 3; ++i) {
+    P4EST_ASSERT (0 <= my_axis[i] && my_axis[i] < P4EST_DIM);
+    P4EST_ASSERT (0 <= target_axis[i] && target_axis[i] < P4EST_DIM);
+  }
+#endif
+
+  P4EST_ASSERT (my_axis[0] != my_axis[2]);
+  P4EST_ASSERT (target_axis[0] != target_axis[2]);
+  P4EST_ASSERT (0 <= edge_reverse[0] && edge_reverse[0] < 2);
+  P4EST_ASSERT (0 <= edge_reverse[2] && edge_reverse[2] < 4);
+#ifdef P4_TO_P8
+  P4EST_ASSERT (my_axis[0] != my_axis[1] && my_axis[1] != my_axis[2]);
+  P4EST_ASSERT (target_axis[0] != target_axis[1] &&
+                target_axis[1] != target_axis[2]);
+  P4EST_ASSERT (0 <= edge_reverse[1] && edge_reverse[1] < 2);
+#else
+  P4EST_ASSERT (my_axis[1] == 0 && target_axis[1] == 0);
+  P4EST_ASSERT (edge_reverse[1] == 0);
+#endif
+  P4EST_ASSERT (coords_in != coords_out);
+
+  for (int d = 0; d < P4EST_DIM; d++) {
+    my_xyz[d] = &coords_in[d];
+    target_xyz[d] = &coords_out[d];
+  }
+
+  *target_xyz[target_axis[0]] =
+    !edge_reverse[0] ? *my_xyz[my_axis[0]] : P4EST_ROOT_LEN -
+    *my_xyz[my_axis[0]];
+#ifdef P4_TO_P8
+  *target_xyz[target_axis[1]] =
+    !edge_reverse[1] ? *my_xyz[my_axis[1]] : P4EST_ROOT_LEN -
+    *my_xyz[my_axis[1]];
+#endif
+  switch (edge_reverse[2]) {
+  case 0:
+    *target_xyz[target_axis[2]] = -*my_xyz[my_axis[2]];
+    break;
+  case 1:
+    *target_xyz[target_axis[2]] = *my_xyz[my_axis[2]] + P4EST_ROOT_LEN;
+    break;
+  case 2:
+    *target_xyz[target_axis[2]] = *my_xyz[my_axis[2]] - P4EST_ROOT_LEN;
+    break;
+  case 3:
+    *target_xyz[target_axis[2]] =
+      P4EST_ROOT_LEN + (P4EST_ROOT_LEN - *my_xyz[my_axis[2]]);
+    break;
+  default:
+    SC_ABORT_NOT_REACHED ();
+  }
+}
+
 void
 p4est_quadrant_transform_face (const p4est_quadrant_t * q,
                                p4est_quadrant_t * r, const int ftransform[])
@@ -1731,7 +2240,7 @@ p4est_quadrant_linear_id (const p4est_quadrant_t * quadrant, int level)
 #endif
 
   P4EST_ASSERT (p4est_quadrant_is_extended (quadrant));
-  P4EST_ASSERT ((int) quadrant->level >= level && level >= 0);
+  P4EST_ASSERT (0 <= level && level <= P4EST_OLD_MAXLEVEL);
 
   /* this preserves the high bits from negative numbers */
   x = quadrant->x >> (P4EST_MAXLEVEL - level);
@@ -1752,16 +2261,47 @@ p4est_quadrant_linear_id (const p4est_quadrant_t * quadrant, int level)
   return id;
 }
 
+void
+p4est_quadrant_linear_id_ext128 (const p4est_quadrant_t *
+                                 quadrant, int level, p4est_lid_t * id)
+{
+  int                 i;
+  uint64_t            x, y;
+#ifdef P4_TO_P8
+  uint64_t            z;
+#endif
+
+  P4EST_ASSERT (p4est_quadrant_is_extended (quadrant));
+  P4EST_ASSERT (0 <= level && level <= P4EST_MAXLEVEL);
+
+  /* this preserves the high bits from negative numbers */
+  x = quadrant->x >> (P4EST_MAXLEVEL - level);
+  y = quadrant->y >> (P4EST_MAXLEVEL - level);
+#ifdef P4_TO_P8
+  z = quadrant->z >> (P4EST_MAXLEVEL - level);
+#endif
+
+  p4est_lid_set_zero (id);
+  for (i = 0; i < level + 2; ++i) {
+    if (x & ((uint64_t) 1 << i))
+      p4est_lid_set_bit (id, P4EST_DIM * i);
+    if (y & ((uint64_t) 1 << i))
+      p4est_lid_set_bit (id, P4EST_DIM * i + 1);
+#ifdef P4_TO_P8
+    if (z & ((uint64_t) 1 << i))
+      p4est_lid_set_bit (id, P4EST_DIM * i + 2);
+#endif
+  }
+}
+
 void
 p4est_quadrant_set_morton (p4est_quadrant_t * quadrant,
                            int level, uint64_t id)
 {
   int                 i;
 
-  P4EST_ASSERT (0 <= level && level <= P4EST_QMAXLEVEL);
-  if (level < P4EST_QMAXLEVEL) {
-    P4EST_ASSERT (id < ((uint64_t) 1 << P4EST_DIM * (level + 2)));
-  }
+  P4EST_ASSERT (0 <= level && level <= P4EST_OLD_QMAXLEVEL);
+  P4EST_ASSERT (id < ((uint64_t) 1 << P4EST_DIM * (level + 2)));
 
   quadrant->level = (int8_t) level;
   quadrant->x = 0;
@@ -1787,6 +2327,65 @@ p4est_quadrant_set_morton (p4est_quadrant_t * quadrant,
 #ifdef P4_TO_P8
   quadrant->z <<= (P4EST_MAXLEVEL - level);
 
+#if P4EST_MAXLEVEL < 30         /* This is never true on purpose. */
+  SC_ABORT_NOT_REACHED ();
+  /* this was needed when number of bits could be more than MAXLEVEL + 2 */
+  if (quadrant->x >= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 1))
+    quadrant->x -= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 2);
+  if (quadrant->y >= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 1))
+    quadrant->y -= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 2);
+  if (quadrant->z >= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 1))
+    quadrant->z -= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 2);
+#endif
+#endif
+
+  P4EST_ASSERT (p4est_quadrant_is_extended (quadrant));
+}
+
+void
+p4est_quadrant_set_morton_ext128 (p4est_quadrant_t * quadrant,
+                                  int level, const p4est_lid_t * id)
+{
+  int                 i;
+#ifdef P4EST_ENABLE_DEBUG
+  p4est_lid_t         one, temp_lid;
+#endif
+
+  P4EST_ASSERT (0 <= level && level <= P4EST_QMAXLEVEL);
+
+#ifdef P4EST_ENABLE_DEBUG
+  p4est_lid_set_one (&one);
+  p4est_lid_shift_left (&one, P4EST_DIM * (level + 2), &(temp_lid));
+  P4EST_ASSERT (p4est_lid_compare (id, &temp_lid) < 0);
+#endif
+
+  quadrant->level = (int8_t) level;
+  quadrant->x = 0;
+  quadrant->y = 0;
+#ifdef P4_TO_P8
+  quadrant->z = 0;
+#endif
+
+  /* this may set the sign bit to create negative numbers */
+  for (i = 0; i < level + 2; ++i) {
+    if (p4est_lid_chk_bit (id, P4EST_DIM * i))
+      p4est_qcoord_set_bit (&quadrant->x, i);
+
+    if (p4est_lid_chk_bit (id, P4EST_DIM * i + 1))
+      p4est_qcoord_set_bit (&quadrant->y, i);
+
+#ifdef P4_TO_P8
+    if (p4est_lid_chk_bit (id, P4EST_DIM * i + 2))
+      p4est_qcoord_set_bit (&quadrant->z, i);
+#endif
+  }
+
+  quadrant->x <<= (P4EST_MAXLEVEL - level);
+  quadrant->y <<= (P4EST_MAXLEVEL - level);
+#ifdef P4_TO_P8
+  quadrant->z <<= (P4EST_MAXLEVEL - level);
+
+#if P4EST_MAXLEVEL < 30         /* This is never true. */
   /* this is needed whenever the number of bits is more than MAXLEVEL + 2 */
   if (quadrant->x >= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 1))
     quadrant->x -= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 2);
@@ -1794,7 +2393,298 @@ p4est_quadrant_set_morton (p4est_quadrant_t * quadrant,
     quadrant->y -= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 2);
   if (quadrant->z >= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 1))
     quadrant->z -= (p4est_qcoord_t) 1 << (P4EST_MAXLEVEL + 2);
+#endif
+
 #endif
 
   P4EST_ASSERT (p4est_quadrant_is_extended (quadrant));
 }
+
+void
+p4est_quadrant_successor (const p4est_quadrant_t * quadrant,
+                          p4est_quadrant_t * result)
+{
+  int                 level;
+  int                 successor_id;
+  p4est_qcoord_t      quadrant_length, mask;
+
+  P4EST_ASSERT (p4est_quadrant_is_extended (quadrant));
+  P4EST_ASSERT (quadrant->level > 0);
+
+  successor_id =
+    p4est_quadrant_ancestor_id (quadrant, level = quadrant->level) + 1;
+
+  /* iterate until it is possible to increment the child/ancestor_id */
+  while (successor_id == P4EST_CHILDREN) {
+    successor_id = p4est_quadrant_ancestor_id (quadrant, --level) + 1;
+    P4EST_ASSERT (level > 0);
+  }
+  P4EST_ASSERT (0 < successor_id && successor_id < P4EST_CHILDREN);
+
+  /* compute result */
+  if (level < quadrant->level) {
+    quadrant_length = P4EST_QUADRANT_LEN (level);
+    mask = ~(P4EST_QUADRANT_LEN (level - 1) - 1);
+
+    /* coarsen to level - 1 and add shifts according to the successor_id */
+    result->x =
+      (successor_id & 0x01) ? (quadrant->x & mask) +
+      quadrant_length : quadrant->x & mask;
+    result->y =
+      (successor_id & 0x02) ? (quadrant->y & mask) +
+      quadrant_length : quadrant->y & mask;
+#ifdef P4_TO_P8
+    result->z =
+      (successor_id & 0x04) ? (quadrant->z & mask) +
+      quadrant_length : quadrant->z & mask;
+#endif
+    result->level = quadrant->level;
+  }
+  else {
+    p4est_quadrant_sibling (quadrant, result, successor_id);
+  }
+  P4EST_ASSERT (p4est_quadrant_is_extended (result));
+}
+
+void
+p4est_quadrant_predecessor (const p4est_quadrant_t * quadrant,
+                            p4est_quadrant_t * result)
+{
+  int                 level;
+  int                 predecessor_id;
+  p4est_qcoord_t      quadrant_length, mask;
+  p4est_qcoord_t      shift_last_descendant, shift_combined;
+
+  P4EST_ASSERT (p4est_quadrant_is_extended (quadrant));
+  P4EST_ASSERT (quadrant->level > 0);
+
+  predecessor_id =
+    p4est_quadrant_ancestor_id (quadrant, level = quadrant->level) - 1;
+
+  /* iterate until it is possible to decrement the child/ancestor_id */
+  while (predecessor_id == -1) {
+    predecessor_id = p4est_quadrant_ancestor_id (quadrant, --level) - 1;
+    P4EST_ASSERT (level > 0);
+  }
+  P4EST_ASSERT (0 <= predecessor_id && predecessor_id < P4EST_CHILDREN - 1);
+
+  /* compute result */
+  if (level < quadrant->level) {
+    quadrant_length = P4EST_QUADRANT_LEN (level);
+    shift_last_descendant =
+      quadrant_length - P4EST_QUADRANT_LEN (quadrant->level);
+    shift_combined = shift_last_descendant + quadrant_length;
+    mask = ~(P4EST_QUADRANT_LEN (level - 1) - 1);
+
+    /* Coarsen to level - 1 and add shifts according to the predecessor_id
+     * and determine the last descendant of the resulting quadrant
+     * of level ´level´ on level quadrant->level.
+     */
+    result->x =
+      (predecessor_id & 0x01) ? (quadrant->x & mask) +
+      shift_combined : (quadrant->x & mask) + shift_last_descendant;
+    result->y =
+      (predecessor_id & 0x02) ? (quadrant->y & mask) +
+      shift_combined : (quadrant->y & mask) + shift_last_descendant;
+#ifdef P4_TO_P8
+    result->z =
+      (predecessor_id & 0x04) ? (quadrant->z & mask) +
+      shift_combined : (quadrant->z & mask) + shift_last_descendant;
+#endif
+    result->level = quadrant->level;
+  }
+  else {
+    p4est_quadrant_sibling (quadrant, result, predecessor_id);
+  }
+  P4EST_ASSERT (p4est_quadrant_is_extended (result));
+}
+
+void
+p4est_quadrant_srand (const p4est_quadrant_t * q, sc_rand_state_t * rstate)
+{
+  P4EST_ASSERT (p4est_quadrant_is_valid (q));
+  P4EST_ASSERT (rstate != NULL);
+#ifdef P4_TO_P8
+  p4est_lid_t         id;
+#endif
+
+  if (q->level <= P4EST_OLD_QMAXLEVEL) {
+    *rstate = p4est_quadrant_linear_id (q, P4EST_OLD_QMAXLEVEL);
+  }
+  else {
+#ifndef P4_TO_P8
+    /* this case never occurs on purpose in 2D */
+    SC_ABORT_NOT_REACHED ();
+#else
+    p4est_quadrant_linear_id_ext128 (q, P4EST_QMAXLEVEL, &id);
+    *rstate = id.high_bits ^ id.low_bits;
+#endif
+  }
+}
+
+void
+p4est_neighbor_transform_quadrant (const p4est_neighbor_transform_t * nt,
+                                   const p4est_quadrant_t * self_quad,
+                                   p4est_quadrant_t * neigh_quad)
+{
+  p4est_qcoord_t      self_from_origin[2][P4EST_DIM];
+  p4est_qcoord_t      neigh_from_origin[2][P4EST_DIM];
+  p4est_qcoord_t      h = P4EST_QUADRANT_LEN (self_quad->level);
+
+  self_from_origin[0][0] = self_quad->x - nt->origin_self[0];
+  self_from_origin[0][1] = self_quad->y - nt->origin_self[1];
+#ifdef P4_TO_P8
+  self_from_origin[0][2] = self_quad->z - nt->origin_self[2];
+#endif
+
+  for (int d = 0; d < P4EST_DIM; d++) {
+    self_from_origin[1][d] = self_from_origin[0][d] + h;
+  }
+
+  for (int i = 0; i < 2; i++) {
+    for (int d = 0; d < P4EST_DIM; d++) {
+      neigh_from_origin[i][d] =
+        nt->sign[d] * self_from_origin[i][nt->perm[d]];
+    }
+  }
+
+  neigh_quad->x =
+    SC_MIN (neigh_from_origin[0][0],
+            neigh_from_origin[1][0]) + nt->origin_neighbor[0];
+  neigh_quad->y =
+    SC_MIN (neigh_from_origin[0][1],
+            neigh_from_origin[1][1]) + nt->origin_neighbor[1];
+#ifdef P4_TO_P8
+  neigh_quad->z =
+    SC_MIN (neigh_from_origin[0][2],
+            neigh_from_origin[1][2]) + nt->origin_neighbor[2];
+#endif
+  neigh_quad->level = self_quad->level;
+}
+
+void
+p4est_neighbor_transform_quadrant_reverse (const p4est_neighbor_transform_t *
+                                           nt,
+                                           const p4est_quadrant_t *
+                                           neigh_quad,
+                                           p4est_quadrant_t * self_quad)
+{
+  p4est_qcoord_t      neigh_from_origin[2][P4EST_DIM];
+  p4est_qcoord_t      self_from_origin[2][P4EST_DIM];
+  p4est_qcoord_t      h = P4EST_QUADRANT_LEN (neigh_quad->level);
+
+  neigh_from_origin[0][0] = neigh_quad->x - nt->origin_neighbor[0];
+  neigh_from_origin[0][1] = neigh_quad->y - nt->origin_neighbor[1];
+#ifdef P4_TO_P8
+  neigh_from_origin[0][2] = neigh_quad->z - nt->origin_neighbor[2];
+#endif
+
+  for (int d = 0; d < P4EST_DIM; d++) {
+    neigh_from_origin[1][d] = neigh_from_origin[0][d] + h;
+  }
+
+  for (int i = 0; i < 2; i++) {
+    for (int d = 0; d < P4EST_DIM; d++) {
+      self_from_origin[i][nt->perm[d]] =
+        nt->sign[d] * neigh_from_origin[i][d];
+    }
+  }
+
+  self_quad->x =
+    SC_MIN (self_from_origin[0][0],
+            self_from_origin[1][0]) + nt->origin_self[0];
+  self_quad->y =
+    SC_MIN (self_from_origin[0][1],
+            self_from_origin[1][1]) + nt->origin_self[1];
+#ifdef P4_TO_P8
+  self_quad->z =
+    SC_MIN (self_from_origin[0][2],
+            self_from_origin[1][2]) + nt->origin_self[2];
+#endif
+  self_quad->level = neigh_quad->level;
+}
+
+int
+p4est_quadrant_is_ancestor_face (const p4est_quadrant_t * descendant,
+                                 const p4est_quadrant_t * ancestor, int face)
+{
+  p4est_qcoord_t      dx, ax;
+
+  P4EST_ASSERT (p4est_quadrant_is_valid (descendant));
+  P4EST_ASSERT (p4est_quadrant_is_valid (ancestor));
+  P4EST_ASSERT (p4est_quadrant_is_ancestor (ancestor, descendant));
+  P4EST_ASSERT (0 <= face && face < P4EST_FACES);
+
+  switch (face >> 1) {
+  case 0:
+    dx = descendant->x;
+    ax = ancestor->x;
+    break;
+  case 1:
+    dx = descendant->y;
+    ax = ancestor->y;
+    break;
+#ifdef P4_TO_P8
+  case 2:
+    dx = descendant->z;
+    ax = ancestor->z;
+    break;
+#endif
+  default:
+    SC_ABORT_NOT_REACHED ();
+  }
+  if (face & 0x01) {
+    dx += P4EST_QUADRANT_LEN (descendant->level);
+    ax += P4EST_QUADRANT_LEN (ancestor->level);
+  }
+
+  return dx == ax;
+}
+
+int
+p4est_quadrant_is_ancestor_corner (const p4est_quadrant_t * descendant,
+                                   const p4est_quadrant_t * ancestor,
+                                   int corner)
+{
+  p4est_qcoord_t      ax, ay, dx, dy, al, dl;
+#ifdef P4_TO_P8
+  p4est_qcoord_t      az, dz;
+#endif
+
+  P4EST_ASSERT (p4est_quadrant_is_valid (ancestor));
+  P4EST_ASSERT (p4est_quadrant_is_valid (descendant));
+  P4EST_ASSERT (p4est_quadrant_is_ancestor (ancestor, descendant));
+  P4EST_ASSERT (0 <= corner && corner < P4EST_CHILDREN);
+
+  ax = ancestor->x;
+  ay = ancestor->y;
+  al = P4EST_QUADRANT_LEN (ancestor->level);
+  dx = descendant->x;
+  dy = descendant->y;
+  dl = P4EST_QUADRANT_LEN (descendant->level);
+#ifdef P4_TO_P8
+  az = ancestor->z;
+  dz = descendant->z;
+#endif
+
+  if (corner & 0x01) {
+    ax += al;
+    dx += dl;
+  }
+  if (corner & 0x02) {
+    ay += al;
+    dy += dl;
+  }
+#ifdef P4_TO_P8
+  if (corner & 0x04) {
+    az += al;
+    dz += dl;
+  }
+#endif
+
+  return (ax == dx && ay == dy
+#ifdef P4_TO_P8
+          && az == dz
+#endif
+    );
+}
