@@ -33,9 +33,15 @@
 #define P4EST_BITS_H
 
 #include <p4est.h>
+#include <sc_random.h>
 
 SC_EXTERN_C_BEGIN;
 
+/** Write -1 into the pad8 and pad16 members of a quadrant.
+ * This helps with valgrind cleanliness if a quadrant is sent over MPI.
+ */
+void                p4est_quadrant_pad (p4est_quadrant_t * q);
+
 /** Prints one line with quadrant's x, y and level.
  * \param [in] log_priority  see \ref logpriorities in sc.h for the meanings
  *                           of numerical priority values
@@ -50,6 +56,14 @@ void                p4est_quadrant_print (int log_priority,
 int                 p4est_quadrant_is_equal (const p4est_quadrant_t * q1,
                                              const p4est_quadrant_t * q2);
 
+/** Copy the Morton indices of the quadrant \a q.
+ *  \param[in] q      	 An extended quadrant.
+ *  \param[in,out] copy  An existing quadrant that Morton indices will
+ *                       be set to the Morton indices of \a q.
+ */
+void                p4est_quadrant_copy (const p4est_quadrant_t * q,
+                                         p4est_quadrant_t * copy);
+
 /** Test if two quadrants overlap.
  * \return true if \a q1 and \a q2 are equal or one is the ancestor of the
  * other.
@@ -74,6 +88,18 @@ int                 p4est_quadrant_is_equal_piggy (const p4est_quadrant_t *
  */
 int                 p4est_quadrant_compare (const void *v1, const void *v2);
 
+/** Compare two sets of coordinates in their Morton ordering.
+ * Coordinates are signed, but the sorted order will treat them
+ * as unsigned, with negative coordinates being greater than
+ * positive coordinates because of their representation in twos-complement.
+ * \param [in] v1, v2    Two sets of 2d coordinates.
+ * \return Returns < 0 if \a v1 < \a v2,
+ *                   0 if \a v1 == \a v2,
+ *                 > 0 if \a v1 > \a v2
+ */
+int                 p4est_coordinates_compare (const p4est_qcoord_t v1[],
+                                               const p4est_qcoord_t v2[]);
+
 /** Compare two quadrants in their Morton ordering, with equivalence if the
  * two quadrants overlap.
  * \return Returns < 0 if \a v1 < \a v2 and \a v1 and \a v2 do not overlap,
@@ -162,6 +188,13 @@ int                 p4est_quadrant_ancestor_id (const p4est_quadrant_t * q,
  */
 int                 p4est_quadrant_child_id (const p4est_quadrant_t * q);
 
+/** Test if Morton indices are inside the unit tree.
+ * \param [in] coord   2d coordinates.
+ * \return Returns true if \a (coord[0],coord[1]) is inside the unit tree.
+ */
+int                 p4est_coordinates_is_inside_root (const p4est_qcoord_t
+                                                      coord[]);
+
 /** Test if a quadrant is inside the unit tree.
  * \param [in] q Quadrant to be tested.
  * \return Returns true if \a q is inside the unit tree.
@@ -198,6 +231,14 @@ int                 p4est_quadrant_is_outside_corner (const p4est_quadrant_t *
 int                 p4est_quadrant_is_node (const p4est_quadrant_t * q,
                                             int inside);
 
+/** Test if Morton indices are valid and are inside the unit tree.
+ * \param [in] coord  2d coordinates.
+ * \param [in] level  level
+ * \return Returns true if \a (coord[0],coord[1],level) is valid.
+ */
+int                 p4est_coordinates_is_valid (const p4est_qcoord_t coord[],
+                                                int level);
+
 /** Test if a quadrant has valid Morton indices and is inside the unit tree.
  * \param [in] q Quadrant to be tested.
  * \return Returns true if \a q is valid.
@@ -300,10 +341,45 @@ int                 p4est_quadrant_is_inside_tree (p4est_tree_t * tree,
                                                    const p4est_quadrant_t *
                                                    q);
 
+/** Whether two descendants of a quadrant are first and last, up to size.
+ * \param [in] f    An extended quadrant, need not be of maximum level.
+ * \param [in] l    An extended quadrant, need not be of maximum level.
+ *                  It must be greater equal \b f in the space filling curve.
+ * \param [in] a    An extended quadrant,
+ *                  equal to or ancestor of \b f, and likewise to/of \b l.
+ * \return          Whether the first corner of \b f equals that of \b a and
+ *                  the last corner of \b l equals that of \b a.
+ */
+int                 p4est_quadrant_is_first_last (const p4est_quadrant_t * f,
+                                                  const p4est_quadrant_t * l,
+                                                  const p4est_quadrant_t * a);
+
+/** Enlarge a quadrant as long as its first corner stays the same.
+ * We limit the enlargement by containing it in an ancestor quadrant.
+ * \param [in] a    Extended quadrant.  On input and output, equal to or
+ *                  strict ancestor of the quadrant \b q to be modified.
+ * \param [in,out] q    On input and output, an extended quadrant and also
+ *                      equal or a strict descendant of \b a.
+ *                      Possibly enlarged by this function.
+ */
+void                p4est_quadrant_enlarge_first (const p4est_quadrant_t * a,
+                                                  p4est_quadrant_t * q);
+
+/** Enlarge a quadrant as long as its last corner stays the same.
+ * We limit the enlargement by containing it in an ancestor quadrant.
+ * \param [in] a    Extended quadrant.  On input and output, equal to or
+ *                  strict ancestor of the quadrant \b q to be modified.
+ * \param [in,out] q    On input and output, an extended quadrant and also
+ *                      equal or a strict descendant of \b a.
+ *                      Possibly enlarged by this function.
+ */
+void                p4est_quadrant_enlarge_last (const p4est_quadrant_t * a,
+                                                 p4est_quadrant_t * q);
+
 /** Compute the ancestor of a quadrant at a given level.
  * \param [in]  q       Input quadrant.
  * \param [in]  level   A smaller level than q.
- * \param [in,out]  r   Existing quadrent whose Morton index will be filled
+ * \param [in,out]  r   Existing quadrant whose Morton index will be filled
  *                      with the ancestor of q at the given level.
  * \note The quadrant q may point to the same quadrant as r.
  *       The user_data of r are never modified.
@@ -354,18 +430,18 @@ void                p4est_quadrant_face_neighbor (const p4est_quadrant_t * q,
 /** Compute the face neighbor of a quadrant, transforming across tree
  * boundaries if necessary.
  * \param [in]     q      Input quadrant, must be valid.
- * \param [in]     t      Tree that contains \q.
+ * \param [in]     t      Tree that contains \a q.
  * \param [in]     face   The face across which to generate the neighbor.
  * \param [in,out] r      Existing quadrant whose Morton index will be filled.
- *                        By convention, if there is no tree across \face,
- *                        \r has the same Morton index as \q.
- * \param [in,out] nface  if not NULL, set to the face of \r that neighbors
- *                        \q.  nface is encoded with orientation information
+ *                        By convention, if there is no tree across \a face,
+ *                        \a r has the same Morton index as \a q.
+ * \param [in,out] nface  if not NULL, set to the face of \a r that neighbors
+ *                        \a q.  nface is encoded with orientation information
  *                        in the same manner as the tree_to_face array in
  *                        the p4est_connectivity_t struct.
  * \param [in]     conn   The connectivity structure for the forest.
- * \return Returns the tree that contains \r.  By convention, if there is no
- * tree across \face, then -1 is returned.
+ * \return Returns the tree that contains \a r.  By convention, if there is no
+ * tree across \a face, then -1 is returned.
  */
 p4est_topidx_t      p4est_quadrant_face_neighbor_extra (const p4est_quadrant_t
                                                         * q, p4est_topidx_t t,
@@ -429,14 +505,14 @@ void                p4est_quadrant_corner_neighbor (const p4est_quadrant_t *
  * boundaries if necessary.  Only computes neighbors that are not face or edge
  * neighbors.
  * \param [in]     q      Input quadrant, must be valid.
- * \param [in]     t      Tree that contains \q.
+ * \param [in]     t      Tree that contains \a q.
  * \param [in]     corner The corner across which to generate the neighbor.
  * \param [in,out] quads  An initialized but empty array where the corner
  *                        neighbors will be placed.
  * \param [in,out] treeids An initialized but empty array where the ids of the
  *                        trees containing the corner neighbors will be placed.
  * \param [in,out] ncorners if not NULL, filled with the corners of \a quads
- *                          that neighbor \q.
+ *                          that neighbor \a q.
  * \param [in]     conn   The connectivity structure for the forest.
  */
 void                p4est_quadrant_corner_neighbor_extra (const
@@ -563,12 +639,27 @@ void                p4est_nearest_common_ancestor_D (const p4est_quadrant_t *
  *             [3,5]        The coordinate axis sequence of the target face.
  *             [6,8]        Edge reverse flag for axis 0; face code for 1.
  *             [1,4,7]      0 (unused for compatibility with 3D).
- * \note \a q and \q r may NOT point to the same quadrant structure.
+ * \note \a q and \a r may NOT point to the same quadrant structure.
  */
 void                p4est_quadrant_transform_face (const p4est_quadrant_t * q,
                                                    p4est_quadrant_t * r,
                                                    const int ftransform[]);
 
+/** Transforms coordinates across a face between trees.
+ * \param [in]  coords_in   Input coordinates.
+ * \param [out] coords_out  Output coordinates.
+ * \param [in] ftransform   This array holds 9 integers.
+ *             [0,2]        The coordinate axis sequence of the origin face.
+ *             [3,5]        The coordinate axis sequence of the target face.
+ *             [6,8]        Edge reverse flag for axis 0; face code for 1.
+ *             [1,4,7]      0 (unused for compatibility with 3D).
+ */
+void                p4est_coordinates_transform_face (const p4est_qcoord_t
+                                                      coords_in[],
+                                                      p4est_qcoord_t
+                                                      coords_out[],
+                                                      const int ftransform[]);
+
 /** Checks if a quadrant touches a corner (diagonally inside or outside).
  */
 int                 p4est_quadrant_touches_corner (const p4est_quadrant_t * q,
@@ -592,23 +683,116 @@ void                p4est_quadrant_shift_corner (const p4est_quadrant_t * q,
                                                  int corner);
 
 /** Computes the linear position of a quadrant in a uniform grid.
- * \param [in] quadrant  Quadrant whose id will be computed.
+ * The grid and quadrant levels need not coincide.
+ * If they do, this is the inverse of \ref p4est_quadrant_set_morton.
+ * \param [in] quadrant  Quadrant whose linear index will be computed.
+ *                       If the quadrant is smaller than the grid (has a higher
+ *                       quadrant->level), the result is computed from its
+ *                       ancestor at the grid's level.
+ *                       If the quadrant has a smaller level than the grid (it
+ *                       is bigger than a grid cell), the grid cell sharing its
+ *                       lower left corner is used as reference.
+ * \param [in] level     The level of the regular grid compared to which the
+ *                       linear position is to be computed.
  * \return Returns the linear position of this quadrant on a grid.
- * \note This is the inverse operation of p4est_quadrant_set_morton.
- *       The user_data of \a quadrant is never modified.
+ * \note The user_data of \a quadrant is never modified.
  */
 uint64_t            p4est_quadrant_linear_id (const p4est_quadrant_t *
                                               quadrant, int level);
 
 /** Set quadrant Morton indices based on linear position in uniform grid.
+ * This is the inverse operation of \ref p4est_quadrant_linear_id.
  * \param [in,out] quadrant  Quadrant whose Morton indices will be set.
- * \param [in]     id        The linear position of this quadrant on a grid.
- * \note This is the inverse operation of p4est_quadrant_linear_id.
- *       The user_data of \a quadrant is never modified.
+ * \param [in]     level     Level of the grid and of the resulting quadrant.
+ * \param [in]     id        Linear index of the quadrant on a uniform grid.
+ * \note The user_data of \a quadrant is never modified.
  */
 void                p4est_quadrant_set_morton (p4est_quadrant_t * quadrant,
                                                int level, uint64_t id);
 
+/** Compute the successor according to the Morton index in a uniform mesh.
+ * \param[in] quadrant  Quadrant whose Morton successor will be computed.
+ *                      Must not be the last (top right) quadrant in the tree.
+ * \param[in,out] result    The coordinates and level of the successor of
+ *                          \b quadrant will be saved in \b result.
+ */
+void                p4est_quadrant_successor (const p4est_quadrant_t *
+                                              quadrant,
+                                              p4est_quadrant_t * result);
+
+/** Compute the predecessor according to the Morton index in a uniform mesh.
+ * \param[in] quadrant  Quadrant whose Morton predecessor will be computed.
+ *                      Must not be the first (bottom left) quadrant in the tree.
+ * \param[in,out] result    The coordinates and level of the predecessor of
+ *                          \b quadrant will be saved in \b result.
+ */
+void                p4est_quadrant_predecessor (const p4est_quadrant_t *
+                                                quadrant,
+                                                p4est_quadrant_t * result);
+
+/** Initialize a random number generator by quadrant coordinates.
+ * This serves to generate partition-independent and reproducible samples.
+ * \param [in]  q               Valid quadrant.
+ * \param [out] rstate          New state of random number generator.
+ */
+void                p4est_quadrant_srand (const p4est_quadrant_t * q,
+                                          sc_rand_state_t * rstate);
+
+/** Transform a quadrant from self's coordinate system to neighbor's coordinate system.
+ *
+ * \param [in]  nt            A neighbor transform.
+ * \param [in]  self_quad     Input quadrant in self coordinates.
+ * \param [out] neigh_coords  Quad transformed into neighbor coordinates.
+ *
+ * \note This transform gives meaningful results when \a self_quad is inside
+ * the tree root or touches the interface between the two trees in the
+ * transform.
+ */
+void                p4est_neighbor_transform_quadrant
+  (const p4est_neighbor_transform_t * nt,
+   const p4est_quadrant_t * self_quad, p4est_quadrant_t * neigh_quad);
+
+/** Transform a quadrant from a neighbors's coordinate system to self's coordinate system.
+ *
+ * \param [in]  nt            A neighbor transform.
+ * \param [in]  neigh_coords  Input quadrant in neighbor coordinates.
+ * \param [out] self_coords   Quad transformed into self coordinates.
+ *
+ * \note This transform gives meaningful results when \a neigh_quad is inside
+ * the tree root or touches the interface between the two trees in the
+ * transform.
+ */
+void                p4est_neighbor_transform_quadrant_reverse
+  (const p4est_neighbor_transform_t * nt,
+   const p4est_quadrant_t * neigh_quad, p4est_quadrant_t * self_quad);
+
+/** Check if a descendant shares a face with a (strict) ancestor.
+ *
+ * \param [in]  descendant   The descendant in question.
+ * \param [in]  ancestor     The ancestor must not be equal to the descendant.
+ * \param [in]  face         The face of the descendant.
+ *
+ * \return true if descendant face touches ancestor face, false otherwise.
+*/
+int                 p4est_quadrant_is_ancestor_face (const p4est_quadrant_t *
+                                                     descendant,
+                                                     const p4est_quadrant_t *
+                                                     ancestor, int face);
+
+/** Check if a descendant shares a corner with a (strict) ancestor.
+ *
+ * \param [in]  descendant   The descendant in question.
+ * \param [in]  ancestor     The ancestor must not be equal to the descendant.
+ * \param [in]  corner       The corner of the descendant.
+ *
+ * \return  true if descendant face touches ancestor corner, false otherwise.
+*/
+int                 p4est_quadrant_is_ancestor_corner (const p4est_quadrant_t
+                                                       * descendant,
+                                                       const p4est_quadrant_t
+                                                       * ancestor,
+                                                       int corner);
+
 SC_EXTERN_C_END;
 
 #endif /* !P4EST_BITS_H */
