@@ -40,7 +40,7 @@ refine_callback (p4est_t * p4est, p4est_topidx_t which_tree,
   const p4est_locidx_t old_counter = pp->inside_counter++;
   const uint8_t       flag = pp->flags[old_counter];
 
-  P4EST_ASSERT (pp->coarsen_delay >= 0);
+  P4EST_ASSERT (pp->params.coarsen_delay >= 0);
   P4EST_ASSERT (0 <= old_counter);
   P4EST_ASSERT (0 <= pp->num_replaced
                 && pp->num_replaced <= pp->num_refine_flags);
@@ -52,7 +52,7 @@ refine_callback (p4est_t * p4est, p4est_topidx_t which_tree,
 
   /* increase quadrant's counter of most recent adaptation */
   /* if refinement actually occurs, it will be reset to zero in all children */
-  if (pp->coarsen_delay && q->p.user_int >= 0) {
+  if (pp->params.coarsen_delay && q->p.user_int >= 0) {
     ++q->p.user_int;
   }
 
@@ -80,16 +80,16 @@ replace_on_refine (p4est_t * p4est, p4est_topidx_t which_tree,
   }
 
   /* reset the counter for most recent adaptation */
-  P4EST_ASSERT (pp->coarsen_delay >= 0);
-  if (pp->coarsen_delay) {
+  P4EST_ASSERT (pp->params.coarsen_delay >= 0);
+  if (pp->params.coarsen_delay) {
     for (k = 0; k < P4EST_CHILDREN; ++k) {
       incoming[k]->p.user_int = 0;
     }
   }
 
   /* pass the replaced quadrants to the user-provided function */
-  if (pp->replace_fn != NULL) {
-    pp->replace_fn (p4est, which_tree,
+  if (pp->params.replace_fn != NULL) {
+    pp->params.replace_fn (p4est, which_tree,
                     num_outgoing, outgoing, num_incoming, incoming);
   }
 }
@@ -102,7 +102,7 @@ coarsen_callback (p4est_t * p4est, p4est_topidx_t which_tree,
   const p4est_locidx_t old_counter = pp->inside_counter++;
   int                 k;
 
-  P4EST_ASSERT (pp->coarsen_delay >= 0);
+  P4EST_ASSERT (pp->params.coarsen_delay >= 0);
 
   /* are we not coarsening at all, just counting? */
   if (q[1] == NULL) {
@@ -115,8 +115,8 @@ coarsen_callback (p4est_t * p4est, p4est_topidx_t which_tree,
       /* coarsening flag was not set */
       return 0;
     }
-    if (pp->coarsen_delay && q[k]->p.user_int >= 0 &&
-        q[k]->p.user_int <= pp->coarsen_delay) {
+    if (pp->params.coarsen_delay && q[k]->p.user_int >= 0 &&
+        q[k]->p.user_int <= pp->params.coarsen_delay) {
       /* most recent adaptation has been too recent */
       return 0;
     }
@@ -135,14 +135,14 @@ replace_on_coarsen (p4est_t * p4est, p4est_topidx_t which_tree,
 {
   p4est_wrap_t       *pp = (p4est_wrap_t *) p4est->user_pointer;
   P4EST_ASSERT (num_incoming == 1 && num_outgoing == P4EST_CHILDREN);
-  P4EST_ASSERT (pp->coarsen_delay > 0);
+  P4EST_ASSERT (pp->params.coarsen_delay > 0);
 
   /* reset most recent adaptation timer */
-  incoming[0]->p.user_int = pp->coarsen_affect ? 0 : -1;
+  incoming[0]->p.user_int = pp->params.coarsen_affect ? 0 : -1;
 
   /* pass the replaced quadrants to the user-provided function */
-  if (pp->replace_fn != NULL) {
-    pp->replace_fn (p4est, which_tree,
+  if (pp->params.replace_fn != NULL) {
+    pp->params.replace_fn (p4est, which_tree,
                     num_outgoing, outgoing, num_incoming, incoming);
   }
 }
@@ -157,7 +157,7 @@ replace_on_balance (p4est_t * p4est, p4est_topidx_t which_tree,
 
   /* this function is called when refinement occurs in balance */
   P4EST_ASSERT (num_outgoing == 1 && num_incoming == P4EST_CHILDREN);
-  P4EST_ASSERT (pp->coarsen_delay > 0);
+  P4EST_ASSERT (pp->params.coarsen_delay > 0);
 
   /* negative value means coarsening is allowed next time */
   for (k = 0; k < P4EST_CHILDREN; ++k) {
@@ -165,23 +165,60 @@ replace_on_balance (p4est_t * p4est, p4est_topidx_t which_tree,
   }
 
   /* pass the replaced quadrants to the user-provided function */
-  if (pp->replace_fn != NULL) {
-    pp->replace_fn (p4est, which_tree,
-                    num_outgoing, outgoing, num_incoming, incoming);
+  if (pp->params.replace_fn != NULL) {
+    pp->params.replace_fn (p4est, which_tree,
+                           num_outgoing, outgoing, num_incoming, incoming);
   }
 }
 
+void
+p4est_wrap_params_init (p4est_wrap_params_t *params)
+{
+  memset (params, 0, sizeof (p4est_wrap_params_t));
+
+  params->hollow = 1;
+  p4est_mesh_params_init (&params->mesh_params);
+  params->replace_fn = NULL;
+  params->coarsen_delay = 0;
+  params->coarsen_affect = 0;
+  params->partition_for_coarsening = 1;
+  params->user_pointer = NULL;
+}
+
 p4est_wrap_t       *
 p4est_wrap_new_conn (sc_MPI_Comm mpicomm, p4est_connectivity_t * conn,
                      int initial_level)
 {
-  return p4est_wrap_new_ext (mpicomm, conn, initial_level,
-                             0, P4EST_CONNECT_FULL, NULL, NULL);
+  p4est_wrap_params_t params;
+
+  p4est_wrap_params_init (&params);
+  params.hollow = 0;
+  params.mesh_params.btype = P4EST_CONNECT_FULL;
+  params.mesh_params.compute_level_lists = 1;
+  params.mesh_params.compute_tree_index = 1;
+
+  return p4est_wrap_new_params (mpicomm, conn, initial_level, &params);
 }
 
 p4est_wrap_t       *
 p4est_wrap_new_p4est (p4est_t * p4est, int hollow, p4est_connect_type_t btype,
                       p4est_replace_t replace_fn, void *user_pointer)
+{
+  p4est_wrap_params_t params;
+
+  p4est_wrap_params_init (&params);
+  params.hollow = hollow;
+  params.mesh_params.btype = btype;
+  params.mesh_params.compute_level_lists = 1;
+  params.mesh_params.compute_tree_index = 1;
+  params.replace_fn = replace_fn;
+  params.user_pointer = user_pointer;
+
+  return p4est_wrap_new_p4est_params (p4est, &params);
+}
+
+p4est_wrap_t       *
+p4est_wrap_new_p4est_params (p4est_t * p4est, p4est_wrap_params_t * params)
 {
   p4est_wrap_t       *pp;
 
@@ -190,7 +227,14 @@ p4est_wrap_new_p4est (p4est_t * p4est, int hollow, p4est_connect_type_t btype,
 
   pp = P4EST_ALLOC_ZERO (p4est_wrap_t, 1);
 
-  pp->hollow = hollow;
+  /* store wrap parameters in wrap */
+  if (params != NULL) {
+    pp->params = *params;
+    params = NULL;
+  }
+  else {
+    p4est_wrap_params_init (&pp->params);
+  }
 
   sc_refcount_init (&pp->conn_rc, p4est_package_id);
   pp->conn = p4est->connectivity;
@@ -200,19 +244,20 @@ p4est_wrap_new_p4est (p4est_t * p4est, int hollow, p4est_connect_type_t btype,
   pp->p4est_half = P4EST_HALF;
   pp->p4est_faces = P4EST_FACES;
   pp->p4est_children = P4EST_CHILDREN;
-  pp->btype = btype;
-  pp->replace_fn = replace_fn;
   pp->p4est = p4est;
   pp->weight_exponent = 0;      /* keep this even though using ALLOC_ZERO */
 
-  if (!pp->hollow) {
+  if (!pp->params.hollow) {
     pp->flags = P4EST_ALLOC_ZERO (uint8_t, pp->p4est->local_num_quadrants);
-    pp->ghost = p4est_ghost_new (pp->p4est, pp->btype);
-    pp->mesh = p4est_mesh_new_ext (pp->p4est, pp->ghost, 1, 1, pp->btype);
+    pp->ghost = p4est_ghost_new (pp->p4est, pp->params.mesh_params.btype);
+    pp->mesh =
+      p4est_mesh_new_params (pp->p4est, pp->ghost, &pp->params.mesh_params);
   }
 
+  /* reset the data size since changing the p4est_wrap will affect p.user_int */
+  p4est_reset_data (pp->p4est, 0, NULL, NULL);
+
   pp->p4est->user_pointer = pp;
-  pp->user_pointer = user_pointer;
 
   return pp;
 }
@@ -222,12 +267,27 @@ p4est_wrap_new_ext (sc_MPI_Comm mpicomm, p4est_connectivity_t * conn,
                     int initial_level, int hollow, p4est_connect_type_t btype,
                     p4est_replace_t replace_fn, void *user_pointer)
 {
-  P4EST_ASSERT (p4est_connectivity_is_valid (conn));
+  p4est_wrap_params_t params;
 
-  return p4est_wrap_new_p4est (p4est_new_ext (mpicomm, conn,
-                                              0, initial_level, 1,
-                                              0, NULL, NULL),
-                               hollow, btype, replace_fn, user_pointer);
+  p4est_wrap_params_init (&params);
+  params.hollow = hollow;
+  params.mesh_params.btype = btype;
+  params.mesh_params.compute_level_lists = 1;
+  params.mesh_params.compute_tree_index = 1;
+  params.replace_fn = replace_fn;
+  params.user_pointer = user_pointer;
+
+  return p4est_wrap_new_params (mpicomm, conn, initial_level, &params);
+}
+
+p4est_wrap_t       *
+p4est_wrap_new_params (sc_MPI_Comm mpicomm, p4est_connectivity_t * conn,
+                       int initial_level, p4est_wrap_params_t * params)
+{
+  P4EST_ASSERT (p4est_connectivity_is_valid (conn));
+  return p4est_wrap_new_p4est_params (p4est_new_ext (mpicomm, conn,
+                                                     0, initial_level, 1,
+                                                     0, NULL, NULL), params);
 }
 
 p4est_wrap_t       *
@@ -240,7 +300,9 @@ p4est_wrap_new_copy (p4est_wrap_t * source, size_t data_size,
 
   pp = P4EST_ALLOC_ZERO (p4est_wrap_t, 1);
 
-  pp->hollow = 1;
+  /* copy the sources wrap paramters; however the copy will is hollow */
+  pp->params = source->params;
+  pp->params.hollow = 1;
 
   sc_refcount_init_invalid (&pp->conn_rc);
   pp->conn_owner = (source->conn_owner != NULL ? source->conn_owner : source);
@@ -251,8 +313,7 @@ p4est_wrap_new_copy (p4est_wrap_t * source, size_t data_size,
   pp->p4est_half = P4EST_HALF;
   pp->p4est_faces = P4EST_FACES;
   pp->p4est_children = P4EST_CHILDREN;
-  pp->btype = source->btype;
-  pp->replace_fn = replace_fn;
+  pp->params.replace_fn = replace_fn;
   pp->p4est = p4est_copy (source->p4est, 0);
   if (data_size > 0) {
     p4est_reset_data (pp->p4est, data_size, NULL, NULL);
@@ -261,7 +322,7 @@ p4est_wrap_new_copy (p4est_wrap_t * source, size_t data_size,
   pp->weight_exponent = 0;      /* keep this even though using ALLOC_ZERO */
 
   pp->p4est->user_pointer = pp;
-  pp->user_pointer = user_pointer;
+  pp->params.user_pointer = user_pointer;
 
   return pp;
 }
@@ -324,10 +385,10 @@ p4est_wrap_new_cubed (sc_MPI_Comm mpicomm, int initial_level)
 }
 
 p4est_wrap_t       *
-p4est_wrap_new_disk (sc_MPI_Comm mpicomm, int initial_level)
+p4est_wrap_new_disk (sc_MPI_Comm mpicomm, int px, int py, int initial_level)
 {
-  return p4est_wrap_new_conn (mpicomm,
-                              p4est_connectivity_new_disk (), initial_level);
+  return p4est_wrap_new_conn
+    (mpicomm, p4est_connectivity_new_disk (px, py), initial_level);
 }
 
 #else
@@ -396,7 +457,7 @@ p4est_wrap_destroy (p4est_wrap_t * pp)
     p4est_ghost_destroy (pp->ghost_aux);
   }
 
-  if (!pp->hollow) {
+  if (!pp->params.hollow) {
     p4est_mesh_destroy (pp->mesh);
     p4est_ghost_destroy (pp->ghost);
   }
@@ -425,7 +486,7 @@ void
 p4est_wrap_set_hollow (p4est_wrap_t * pp, int hollow)
 {
   /* Verify consistency */
-  if (!pp->hollow) {
+  if (!pp->params.hollow) {
     P4EST_ASSERT (pp->flags != NULL);
     P4EST_ASSERT (pp->ghost != NULL);
     P4EST_ASSERT (pp->mesh != NULL);
@@ -443,15 +504,16 @@ p4est_wrap_set_hollow (p4est_wrap_t * pp, int hollow)
   P4EST_ASSERT (pp->mesh_aux == NULL);
 
   /* Do nothing if the status is right */
-  if (hollow == pp->hollow) {
+  if (hollow == pp->params.hollow) {
     return;
   }
 
-  if (pp->hollow) {
+  if (pp->params.hollow) {
     /* Allocate the ghost, mesh, and flag members */
     pp->flags = P4EST_ALLOC_ZERO (uint8_t, pp->p4est->local_num_quadrants);
-    pp->ghost = p4est_ghost_new (pp->p4est, pp->btype);
-    pp->mesh = p4est_mesh_new_ext (pp->p4est, pp->ghost, 1, 1, pp->btype);
+    pp->ghost = p4est_ghost_new (pp->p4est, pp->params.mesh_params.btype);
+    pp->mesh =
+      p4est_mesh_new_params (pp->p4est, pp->ghost, &pp->params.mesh_params);
   }
   else {
     /* Free and nullify the ghost, mesh, and flag members */
@@ -463,7 +525,7 @@ p4est_wrap_set_hollow (p4est_wrap_t * pp, int hollow)
     pp->flags = NULL;
   }
   pp->num_refine_flags = pp->inside_counter = pp->num_replaced = 0;
-  pp->hollow = hollow;
+  pp->params.hollow = hollow;
 }
 
 void
@@ -480,8 +542,8 @@ p4est_wrap_set_coarsen_delay (p4est_wrap_t * pp,
   P4EST_ASSERT (pp != NULL);
   P4EST_ASSERT (coarsen_delay >= 0);
 
-  pp->coarsen_delay = coarsen_delay;
-  pp->coarsen_affect = coarsen_affect;
+  pp->params.coarsen_delay = coarsen_delay;
+  pp->params.coarsen_affect = coarsen_affect;
   p4est = pp->p4est;
   P4EST_ASSERT (p4est->data_size == 0);
 
@@ -496,10 +558,17 @@ p4est_wrap_set_coarsen_delay (p4est_wrap_t * pp,
   }
 }
 
+void
+p4est_wrap_set_partitioning (p4est_wrap_t *pp, int partition_for_coarsening)
+{
+  P4EST_ASSERT (pp != NULL);
+  pp->params.partition_for_coarsening = partition_for_coarsening;
+}
+
 p4est_ghost_t      *
 p4est_wrap_get_ghost (p4est_wrap_t * pp)
 {
-  P4EST_ASSERT (!pp->hollow);
+  P4EST_ASSERT (!pp->params.hollow);
 
   return pp->match_aux ? pp->ghost_aux : pp->ghost;
 }
@@ -507,7 +576,7 @@ p4est_wrap_get_ghost (p4est_wrap_t * pp)
 p4est_mesh_t       *
 p4est_wrap_get_mesh (p4est_wrap_t * pp)
 {
-  P4EST_ASSERT (!pp->hollow);
+  P4EST_ASSERT (!pp->params.hollow);
 
   return pp->match_aux ? pp->mesh_aux : pp->mesh;
 }
@@ -521,7 +590,7 @@ p4est_wrap_mark_refine (p4est_wrap_t * pp,
   p4est_locidx_t      pos;
   uint8_t             flag;
 
-  P4EST_ASSERT (!pp->hollow);
+  P4EST_ASSERT (!pp->params.hollow);
   P4EST_ASSERT (p4est->first_local_tree <= which_tree);
   P4EST_ASSERT (which_tree <= p4est->last_local_tree);
 
@@ -548,7 +617,7 @@ p4est_wrap_mark_coarsen (p4est_wrap_t * pp,
   p4est_locidx_t      pos;
   uint8_t             flag;
 
-  P4EST_ASSERT (!pp->hollow);
+  P4EST_ASSERT (!pp->params.hollow);
   P4EST_ASSERT (p4est->first_local_tree <= which_tree);
   P4EST_ASSERT (which_tree <= p4est->last_local_tree);
 
@@ -570,14 +639,16 @@ int
 p4est_wrap_adapt (p4est_wrap_t * pp)
 {
   int                 changed;
+  int                 have_zlib;
 #ifdef P4EST_ENABLE_DEBUG
   p4est_locidx_t      jl, local_num;
 #endif
-  p4est_gloidx_t      global_num;
+  p4est_gloidx_t      global_num, global_num_entry;
+  unsigned            checksum_entry, checksum_exit;
   p4est_t            *p4est = pp->p4est;
 
-  P4EST_ASSERT (!pp->hollow);
-  P4EST_ASSERT (pp->coarsen_delay >= 0);
+  P4EST_ASSERT (!pp->params.hollow);
+  P4EST_ASSERT (pp->params.coarsen_delay >= 0);
 
   P4EST_ASSERT (pp->mesh != NULL);
   P4EST_ASSERT (pp->ghost != NULL);
@@ -594,6 +665,13 @@ p4est_wrap_adapt (p4est_wrap_t * pp)
                                      (P4EST_CHILDREN - 1) *
                                      pp->num_refine_flags);
 
+
+  if ((have_zlib = p4est_have_zlib())) {
+    /* store p4est checksum on entry to compare with results after balancing */
+    global_num_entry = p4est->global_num_quadrants;
+    checksum_entry = p4est_checksum (p4est);
+  }
+
   /* Execute refinement */
   pp->inside_counter = pp->num_replaced = 0;
 #ifdef P4EST_ENABLE_DEBUG
@@ -613,7 +691,8 @@ p4est_wrap_adapt (p4est_wrap_t * pp)
 #endif
   global_num = p4est->global_num_quadrants;
   p4est_coarsen_ext (p4est, 0, 1, coarsen_callback, NULL,
-                     pp->coarsen_delay ? replace_on_coarsen : pp->replace_fn);
+                     pp->params.coarsen_delay ? replace_on_coarsen :
+                     pp->params.replace_fn);
   P4EST_ASSERT (pp->inside_counter == local_num);
   P4EST_ASSERT (local_num - p4est->local_num_quadrants ==
                 pp->num_replaced * (P4EST_CHILDREN - 1));
@@ -625,14 +704,31 @@ p4est_wrap_adapt (p4est_wrap_t * pp)
 
   /* Only if refinement and/or coarsening happened do we need to balance */
   if (changed) {
-    P4EST_FREE (pp->flags);
-    p4est_balance_ext (p4est, pp->btype, NULL, pp->coarsen_delay ?
-                       replace_on_balance : pp->replace_fn);
-    pp->flags = P4EST_ALLOC_ZERO (uint8_t, p4est->local_num_quadrants);
+    p4est_balance_ext (p4est, pp->params.mesh_params.btype, NULL,
+                       pp->params.coarsen_delay ? replace_on_balance :
+                       pp->params.replace_fn);
 
-    pp->ghost_aux = p4est_ghost_new (p4est, pp->btype);
-    pp->mesh_aux = p4est_mesh_new_ext (p4est, pp->ghost_aux, 1, 1, pp->btype);
-    pp->match_aux = 1;
+    /* check if coarsening and balancing canceled out */
+    if (have_zlib && (global_num_entry == p4est->global_num_quadrants)) {
+      /* only compute another checksum for unchanged global quadrant counts */
+      checksum_exit = p4est_checksum (p4est);
+      changed = (checksum_entry != checksum_exit);
+    }
+
+    if (changed) {
+      /* compute new ghost and mesh for the changed p4est */
+      P4EST_FREE (pp->flags);
+      pp->flags = P4EST_ALLOC_ZERO (uint8_t, p4est->local_num_quadrants);
+
+      pp->ghost_aux = p4est_ghost_new (p4est, pp->params.mesh_params.btype);
+      pp->mesh_aux =
+        p4est_mesh_new_params (p4est, pp->ghost_aux, &pp->params.mesh_params);
+      pp->match_aux = 1;
+    }
+    else {
+      memset (pp->flags, 0,
+              sizeof (uint8_t) * pp->p4est->local_num_quadrants);
+    }
   }
 #ifdef P4EST_ENABLE_DEBUG
   else {
@@ -714,7 +810,7 @@ p4est_wrap_partition (p4est_wrap_t * pp, int weight_exponent,
   p4est_gloidx_t      pre_me, pre_next;
   p4est_gloidx_t      post_me, post_next;
 
-  P4EST_ASSERT (!pp->hollow);
+  P4EST_ASSERT (!pp->params.hollow);
 
   P4EST_ASSERT (pp->ghost != NULL);
   P4EST_ASSERT (pp->mesh != NULL);
@@ -746,15 +842,16 @@ p4est_wrap_partition (p4est_wrap_t * pp, int weight_exponent,
   P4EST_ASSERT (weight_exponent == 0 || weight_exponent == 1);
   pp->weight_exponent = weight_exponent;
   changed =
-    p4est_partition_ext (pp->p4est, 1,
+    p4est_partition_ext (pp->p4est, pp->params.partition_for_coarsening,
                          weight_exponent ? partition_weight : NULL) > 0;
 
   if (changed) {
     P4EST_FREE (pp->flags);
     pp->flags = P4EST_ALLOC_ZERO (uint8_t, pp->p4est->local_num_quadrants);
 
-    pp->ghost = p4est_ghost_new (pp->p4est, pp->btype);
-    pp->mesh = p4est_mesh_new_ext (pp->p4est, pp->ghost, 1, 1, pp->btype);
+    pp->ghost = p4est_ghost_new (pp->p4est, pp->params.mesh_params.btype);
+    pp->mesh =
+      p4est_mesh_new_params (pp->p4est, pp->ghost, &pp->params.mesh_params);
 
     /* Query the window onto global quadrant sequence after partition */
     if (unchanged_first != NULL || unchanged_length != NULL ||
@@ -785,7 +882,7 @@ p4est_wrap_partition (p4est_wrap_t * pp, int weight_exponent,
 void
 p4est_wrap_complete (p4est_wrap_t * pp)
 {
-  P4EST_ASSERT (!pp->hollow);
+  P4EST_ASSERT (!pp->params.hollow);
 
   P4EST_ASSERT (pp->ghost != NULL);
   P4EST_ASSERT (pp->mesh != NULL);
