@@ -24,7 +24,36 @@
 
 /** \file p8est_connectivity.h
  *
- * The coarse topological description of the forest.
+ * The connectivity defines the coarse topology of the forest.
+ *
+ * A 3D forest consists of one or more octrees, each of which a logical
+ * cube.
+ * Each tree has a local coordinate system, which defines the origin and the
+ * direction of its x-, y-, and z-axes as well as the numbering of its
+ * faces, edges, and corners.
+ * Each tree may connect to any other tree (including itself) across any of
+ * its faces, edges and/or corners, where the neighbor may be arbitrarily
+ * rotated and/or flipped.
+ * The \ref p8est_connectivity data structure stores these connections.
+ *
+ * We impose the following requirement for consistency of \ref p8est_balance :
+ *
+ * \note If a connectivity implies natural connections between trees that
+ * are edge neighbors without being face neighbors, these edges shall be
+ * encoded explicitly in the connectivity.  If a connectivity implies
+ * natural connections between trees that are corner neighbors without being
+ * edge or face neighbors, these corners shall be encoded explicitly in the
+ * connectivity.
+ * Please see the documentation of \ref p8est_connectivity_t for the exact
+ * encoding convention.
+ *
+ * We provide various predefined connectivities by dedicated constructors,
+ * such as
+ *
+ *  * \ref p8est_connectivity_new_unitcube for the unit square,
+ *  * \ref p8est_connectivity_new_periodic for the periodic unit square,
+ *  * \ref p8est_connectivity_new_brick for a rectangular grid of trees,
+ *  * \ref p8est_connectivity_new_drop for a sparsely loop of trees.
  *
  * \ingroup p8est
  */
@@ -55,13 +84,22 @@ SC_EXTERN_C_BEGIN;
 /** The size of insulation layer */
 #define P8EST_INSUL 27
 
-/* size of face transformation encoding */
+/** Only use logical AND term in 3D */
+#define P8EST_ONLY_P8_LAND(x) && (x)
+
+/** Only use comma and expression in 3D */
+#define P8EST_ONLY_P8_COMMA(x) , (x)
+
+/** Exponentiate with dimension */
+#define P8EST_DIM_POW(a) ((a) * (a) * (a))
+
+/** size of face transformation encoding */
 #define P8EST_FTRANSFORM 9
 
 /** p8est identification string */
 #define P8EST_STRING "p8est"
 
-/* Increase this number whenever the on-disk format for
+/** Increase this number whenever the on-disk format for
  * p8est_connectivity, p8est, or any other 3D data structure changes.
  * The format for reading and writing must be the same.
  */
@@ -80,10 +118,12 @@ SC_EXTERN_C_BEGIN;
 typedef enum
 {
   /* make sure to have different values 2D and 3D */
-  P8EST_CONNECT_FACE = 31,
-  P8EST_CONNECT_EDGE = 32,
-  P8EST_CONNECT_CORNER = 33,
-  P8EST_CONNECT_FULL = P8EST_CONNECT_CORNER
+  P8EST_CONNECT_SELF = 30,      /**< No balance whatsoever. */
+  P8EST_CONNECT_FACE = 31,      /**< Balance across faces only. */
+  P8EST_CONNECT_EDGE = 32,      /**< Balance across faces and edges. */
+  P8EST_CONNECT_ALMOST = P8EST_CONNECT_EDGE,    /**< = CORNER - 1. */
+  P8EST_CONNECT_CORNER = 33,    /**< Balance faces, edges, corners. */
+  P8EST_CONNECT_FULL = P8EST_CONNECT_CORNER     /**< = CORNER. */
 }
 p8est_connect_type_t;
 
@@ -119,6 +159,7 @@ const char         *p8est_connect_type_string (p8est_connect_type_t btype);
  * For faces the order is -x +x -y +y -z +z.
  * They are allocated [0][0]..[0][N-1]..[num_trees-1][0]..[num_trees-1][N-1].
  * where N is 6 for tree and face, 8 for corner, 12 for edge.
+ * If a face is on the physical boundary it must connect to itself.
  *
  * The values for tree_to_face are in 0..23
  * where ttf % 6 gives the face number and ttf / 6 the face orientation code.
@@ -133,8 +174,10 @@ const char         *p8est_connect_type_string (p8est_connect_type_t btype);
  * In this case vertices and tree_to_vertex are set to NULL.
  * Otherwise the vertex coordinates are stored in the array vertices as
  * [0][0]..[0][2]..[num_vertices-1][0]..[num_vertices-1][2].
+ * Vertex coordinates are optional and not used for inferring topology.
  *
- * The edges are only stored when they connect trees.
+ * The edges are stored when they connect trees that are not already face
+ * neighbors at that specific edge.
  * In this case tree_to_edge indexes into \a ett_offset.
  * Otherwise the tree_to_edge entry must be -1 and this edge is ignored.
  * If num_edges == 0, tree_to_edge and edge_to_* arrays are set to NULL.
@@ -147,7 +190,8 @@ const char         *p8est_connect_type_string (p8est_connect_type_t btype);
  * The edge_to_edge array holds values in 0..23, where the lower 12 indicate
  * one edge orientation and the higher 12 the opposite edge orientation.
  *
- * The corners are only stored when they connect trees.
+ * The corners are stored when they connect trees that are not already edge
+ * or face neighbors at that specific corner.
  * In this case tree_to_corner indexes into \a ctt_offset.
  * Otherwise the tree_to_corner entry must be -1 and this corner is ignored.
  * If num_corners == 0, tree_to_corner and corner_to_* arrays are set to NULL.
@@ -159,6 +203,13 @@ const char         *p8est_connect_type_string (p8est_connect_type_t btype);
  * The size of the corner_to_* arrays is num_ctt = ctt_offset[num_corners].
  *
  * The *_to_attr arrays may have arbitrary contents defined by the user.
+ *
+ * \note If a connectivity implies natural connections between trees that
+ * are edge neighbors without being face neighbors, these edges shall be
+ * encoded explicitly in the connectivity.  If a connectivity implies
+ * natural connections between trees that are corner neighbors without being
+ * edge or face neighbors, these corners shall be encoded explicitly in the
+ * connectivity.
  */
 typedef struct p8est_connectivity
 {
@@ -208,34 +259,107 @@ p8est_connectivity_t;
 size_t              p8est_connectivity_memory_used (p8est_connectivity_t *
                                                     conn);
 
+/** Generic interface for transformations between a tree and any of its edge */
 typedef struct
 {
-  p4est_topidx_t      ntree;
-  int8_t              nedge, naxis[3], nflip, corners;
+  p4est_topidx_t      ntree; /**< The number of the tree*/
+  int8_t              nedge; /**< The number of the edge*/
+  int8_t              naxis[3]; /**< The 3 edge coordinate axes*/
+  int8_t              nflip; /**< The orientation of the edge*/
+  int8_t              corners; /**< The corners connected to the edge*/
 }
 p8est_edge_transform_t;
 
+/** Information about the neighbors of an edge*/
 typedef struct
 {
-  int8_t              iedge;
-  sc_array_t          edge_transforms;
+  int8_t              iedge; /**< The information of the edge*/
+  sc_array_t          edge_transforms; /**< The array of neighbors of the originating
+                                         edge */
 }
 p8est_edge_info_t;
 
+/** Generic interface for transformations between a tree and any of its corner */
 typedef struct
 {
-  p4est_topidx_t      ntree;
-  int8_t              ncorner;
+  p4est_topidx_t      ntree; /**< The number of the tree*/
+  int8_t              ncorner; /**< The number of the corner*/
 }
 p8est_corner_transform_t;
 
+/** Information about the neighbors of a corner*/
 typedef struct
 {
-  p4est_topidx_t      icorner;
-  sc_array_t          corner_transforms;
+  p4est_topidx_t      icorner; /**< The number of the originating corner */
+  sc_array_t          corner_transforms; /**< The array of neighbors of the originating
+                                         corner */
 }
 p8est_corner_info_t;
 
+/** Generic interface for transformations between a tree and any of its neighbors */
+typedef struct
+{
+  p8est_connect_type_t neighbor_type; /**< type of connection to neighbor*/
+  p4est_topidx_t      neighbor;   /**< neighbor tree index */
+  int8_t              index_self; /**< index of interface from self's
+                                       perspective */
+  int8_t              index_neighbor; /**< index of interface from neighbor's
+                                           perspective */
+  int8_t              perm[P8EST_DIM]; /**< permutation of dimensions when
+                                            transforming self coords to
+                                            neighbor coords */
+  int8_t              sign[P8EST_DIM]; /**< sign changes when transforming self
+                                            coords to neighbor coords */
+  p4est_qcoord_t      origin_self[P8EST_DIM]; /**< point on the interface from
+                                                  self's perspective */
+  p4est_qcoord_t      origin_neighbor[P8EST_DIM]; /**< point on the interface
+                                                      from neighbor's
+                                                      perspective */
+}
+p8est_neighbor_transform_t;
+
+/* *INDENT-OFF* */
+
+/** Transform from self's coordinate system to neighbor's coordinate system.
+ *
+ * \param [in]  nt            A neighbor transform.
+ * \param [in]  self_coords   Input quadrant coordinates in self coordinates.
+ * \param [out] neigh_coords  Coordinates transformed into neighbor coordinates.
+ */
+void                p8est_neighbor_transform_coordinates
+                      (const p8est_neighbor_transform_t * nt,
+                       const p4est_qcoord_t self_coords[P8EST_DIM],
+                       p4est_qcoord_t neigh_coords[P8EST_DIM]);
+
+/** Transform from neighbor's coordinate system to self's coordinate system.
+ *
+ * \param [in]  nt            A neighbor transform.
+ * \param [in]  neigh_coords  Input quadrant coordinates in self coordinates.
+ * \param [out] self_coords   Coordinates transformed into neighbor coordinates.
+ */
+void                p8est_neighbor_transform_coordinates_reverse
+                      (const p8est_neighbor_transform_t * nt,
+                       const p4est_qcoord_t neigh_coords[P8EST_DIM],
+                       p4est_qcoord_t self_coords[P8EST_DIM]);
+
+/**  Fill an array with the neighbor transforms based on a specific boundary type.
+ *   This function generalizes all other inter-tree transformation objects
+ *
+ * \param [in]  conn   Connectivity structure.
+ * \param [in]  tree_id The number of the tree.
+ * \param [in]  boundary_type  Type of boundary connection (self, face, edge, corner).
+ * \param [in]  boundary_index  The index of the boundary.
+ * \param [in,out] neighbor_transform_array   Array of the neighbor transforms.
+ */
+void                p8est_connectivity_get_neighbor_transforms
+                      (p8est_connectivity_t *conn,
+                       p4est_topidx_t tree_id,
+                       p8est_connect_type_t boundary_type,
+                       int boundary_index,
+                       sc_array_t *neighbor_transform_array);
+
+/* *INDENT-ON* */
+
 /** Store the corner numbers 0..7 for each tree face. */
 extern const int    p8est_face_corners[6][4];
 
@@ -265,37 +389,36 @@ extern const int    p8est_face_edge_permutations[8][4];
 /** Store the 3 occurring sets of 4 permutations per face. */
 extern const int    p8est_face_edge_permutation_sets[3][4];
 
-/** Store the face numbers 0..5 for each tree edge. */
+/** Store the face numbers 0..5 adjacent to each tree edge. */
 extern const int    p8est_edge_faces[12][2];
 
 /** Store the corner numbers 0..8 for each tree edge. */
 extern const int    p8est_edge_corners[12][2];
 
-/** Store the edge corner numbers 0..1 for the corners touching a tree edge */
+/** Store the edge corner numbers 0..1 for the corners touching a tree edge
+    or -1 if combination is invalid */
 extern const int    p8est_edge_edge_corners[12][8];
 
-/** Store the face corner numbers 0..3 for the faces touching a tree edge. */
+/** Store the face corner numbers 0..3 for the faces touching a tree edge.
+    Is -1 for invalid combinations of indices */
 extern const int    p8est_edge_face_corners[12][6][2];
 
-/** Store the face edge numbers 0..3 for the faces touching a tree edge. */
+/** Store the face edge numbers 0..3 for the faces touching a tree edge.
+    Is -1 for invalid combinations of indices */
 extern const int    p8est_edge_face_edges[12][6];
 
-/** Store the sets of permutations that occur */
-extern const int    p8est_edge_corner_permutation_sets[2];
-
-/** Store the two possible permutations that occur */
-extern const int    p8est_edge_corner_permutations[2][2];
-
 /** Store the face numbers 0..5 for each tree corner. */
 extern const int    p8est_corner_faces[8][3];
 
 /** Store the edge numbers 0..11 for each tree corner. */
 extern const int    p8est_corner_edges[8][3];
 
-/** Store the face corner numbers for the faces touching a tree corner. */
+/** Store the face corner numbers for the faces touching a tree corner.
+    Is -1 for invalid combinations. */
 extern const int    p8est_corner_face_corners[8][6];
 
-/** Store the edge corner numbers for the edges touching a tree corner. */
+/** Store the edge corner numbers for the edges touching a tree corner.
+    Is -1 for invalid combinations. */
 extern const int    p8est_corner_edge_corners[8][12];
 
 /** Store the faces for each child and edge, can be -1. */
@@ -311,7 +434,7 @@ extern const int    p8est_child_corner_edges[8][8];
  * It expects a face permutation index that has been precomputed.
  * \param [in] c    A corner number in 0..7.
  * \param [in] f    A face number that touches the corner \a c.
- * \param [in] nf   A neighbor face that is on the other side of \f.
+ * \param [in] nf   A neighbor face that is on the other side of \a f.
  * \param [in] set  A value from \a p8est_face_permutation_sets that is
  *                  obtained using \a f, \a nf, and a valid orientation:
  *                  ref = p8est_face_permutation_refs[f][nf];
@@ -324,51 +447,64 @@ int                 p8est_connectivity_face_neighbor_corner_set
 /** Transform a face corner across one of the adjacent faces into a neighbor tree.
  * This version expects the neighbor face and orientation separately.
  * \param [in] fc   A face corner number in 0..3.
- * \param [in] f    A face number that touches the corner \a c.
- * \param [in] nf   A neighbor face that is on the other side of \f.
- * \param [in] o    The orientation between tree boundary faces \a f and \nf.
+ * \param [in] f    A face that the face corner \a fc is relative to.
+ * \param [in] nf   A neighbor face that is on the other side of \a f.
+ * \param [in] o    The orientation between tree boundary faces \a f and \a nf.
+ * \return          The face corner number relative to the neighbor's face.
  */
-int                 p8est_connectivity_face_neighbor_face_corner_orientation
+int                 p8est_connectivity_face_neighbor_face_corner
   (int fc, int f, int nf, int o);
 
 /** Transform a corner across one of the adjacent faces into a neighbor tree.
  * This version expects the neighbor face and orientation separately.
  * \param [in] c    A corner number in 0..7.
  * \param [in] f    A face number that touches the corner \a c.
- * \param [in] nf   A neighbor face that is on the other side of \f.
- * \param [in] o    The orientation between tree boundary faces \a f and \nf.
+ * \param [in] nf   A neighbor face that is on the other side of \a f.
+ * \param [in] o    The orientation between tree boundary faces \a f and \a nf.
+ * \return          The number of the corner seen from the neighbor tree.
  */
-int                 p8est_connectivity_face_neighbor_corner_orientation
+int                 p8est_connectivity_face_neighbor_corner
   (int c, int f, int nf, int o);
 
+/** Transform a face-edge across one of the adjacent faces into a neighbor tree.
+ * This version expects the neighbor face and orientation separately.
+ * \param [in] fe   A face edge number in 0..3.
+ * \param [in] f    A face number that touches the edge \a e.
+ * \param [in] nf   A neighbor face that is on the other side of \a f.
+ * \param [in] o    The orientation between tree boundary faces \a f and \a nf.
+ * \return          The face edge number seen from the neighbor tree.
+ */
+int                 p8est_connectivity_face_neighbor_face_edge
+  (int fe, int f, int nf, int o);
+
 /** Transform an edge across one of the adjacent faces into a neighbor tree.
  * This version expects the neighbor face and orientation separately.
  * \param [in] e    A edge number in 0..11.
- * \param [in] f    A face number that touches the edge \a e.
- * \param [in] nf   A neighbor face that is on the other side of \f.
- * \param [in] o    The orientation between tree boundary faces \a f and \nf.
+ * \param [in] f    A face 0..5 that touches the edge \a e.
+ * \param [in] nf   A neighbor face that is on the other side of \a f.
+ * \param [in] o    The orientation between tree boundary faces \a f and \a nf.
+ * \return          The edge's number seen from the neighbor.
  */
-int                 p8est_connectivity_face_neighbor_edge_orientation
+int                 p8est_connectivity_face_neighbor_edge
   (int e, int f, int nf, int o);
 
 /** Transform an edge corner across one of the adjacent edges into a neighbor tree.
- * This version expects the neighbor edge and orientation separately.
- * \param [in] ec    An edge corner number in 0..1.
- * \param [in] e    A edge number that touches the corner \a c.
- * \param [in] ne   A neighbor edge that is on the other side of \e.
- * \param [in] o    The orientation between tree boundary faces \a e and \ne.
+ * \param [in] ec   An edge corner number in 0..1.
+ * \param [in] o    The orientation of a tree boundary edge connection.
+ * \return          The edge corner number seen from the other tree.
  */
-int                 p8est_connectivity_edge_neighbor_edge_corner_orientation
-  (int ec, int e, int ne, int o);
+int                 p8est_connectivity_edge_neighbor_edge_corner
+  (int ec, int o);
 
 /** Transform a corner across one of the adjacent edges into a neighbor tree.
  * This version expects the neighbor edge and orientation separately.
  * \param [in] c    A corner number in 0..7.
- * \param [in] e    A edge number that touches the corner \a c.
- * \param [in] ne   A neighbor edge that is on the other side of \e.
- * \param [in] o    The orientation between tree boundary faces \a e and \ne.
+ * \param [in] e    An edge 0..11 that touches the corner \a c.
+ * \param [in] ne   A neighbor edge that is on the other side of \a e.
+ * \param [in] o    The orientation between tree boundary edges \a e and \a ne.
+ * \return          Corner number seen from the neighbor.
  */
-int                 p8est_connectivity_edge_neighbor_corner_orientation
+int                 p8est_connectivity_edge_neighbor_corner
   (int c, int e, int ne, int o);
 
 /** Allocate a connectivity structure.
@@ -394,12 +530,22 @@ p8est_connectivity_t *p8est_connectivity_new (p4est_topidx_t num_vertices,
  * \param [in] num_trees      Number of trees in the forest.
  * \param [in] num_edges      Number of tree-connecting edges.
  * \param [in] num_corners    Number of tree-connecting corners.
+ * \param [in] vertices       Coordinates of the vertices of the trees.
+ * \param [in] ttv            The tree-to-vertex array.
+ * \param [in] ttt            The tree-to-tree array.
+ * \param [in] ttf            The tree-to-face array (int8_t).
+ * \param [in] tte            The tree-to-edge array.
  * \param [in] eoff           Edge-to-tree offsets (num_edges + 1 values).
  *                            This must always be non-NULL; in trivial cases
  *                            it is just a pointer to a p4est_topix value of 0.
+ * \param [in] ett            The edge-to-tree array.
+ * \param [in] ete            The edge-to-edge array.
+ * \param [in] ttc            The tree-to-corner array.
  * \param [in] coff           Corner-to-tree offsets (num_corners + 1 values).
  *                            This must always be non-NULL; in trivial cases
  *                            it is just a pointer to a p4est_topix value of 0.
+ * \param [in] ctt            The corner-to-tree array.
+ * \param [in] ctc            The corner-to-corner array.
  * \return                    The connectivity is checked for validity.
  */
 p8est_connectivity_t *p8est_connectivity_new_copy (p4est_topidx_t
@@ -497,8 +643,10 @@ int                 p8est_connectivity_save (const char *filename,
 p8est_connectivity_t *p8est_connectivity_source (sc_io_source_t * source);
 
 /** Create new connectivity from a memory buffer.
+ * This function aborts on malloc errors.
  * \param [in] buffer   The connectivity is created from this memory buffer.
- * \return              The newly created connectivity, or NULL on error.
+ * \return              The newly created connectivity, or NULL on format
+ *                      error of the buffered connectivity data.
  */
 p8est_connectivity_t *p8est_connectivity_inflate (sc_array_t * buffer);
 
@@ -508,7 +656,7 @@ p8est_connectivity_t *p8est_connectivity_inflate (sc_array_t * buffer);
  * \return              Returns valid connectivity, or NULL on file error.
  */
 p8est_connectivity_t *p8est_connectivity_load (const char *filename,
-                                               size_t * bytes);
+                                               size_t *bytes);
 
 /** Create a connectivity structure for the unit cube.
  */
@@ -524,6 +672,12 @@ p8est_connectivity_t *p8est_connectivity_new_periodic (void);
  */
 p8est_connectivity_t *p8est_connectivity_new_rotwrap (void);
 
+/** Create a connectivity structure for a five-trees geometry with a
+ * hole. The geometry is a 3D extrusion of the two drop example, and
+ * covers [0, 3]*[0, 2]*[0, 3]. The additional dimension is Y.
+ */
+p8est_connectivity_t *p8est_connectivity_new_drop (void);
+
 /** Create a connectivity structure that contains two cubes.
  */
 p8est_connectivity_t *p8est_connectivity_new_twocubes (void);
@@ -570,6 +724,22 @@ p8est_connectivity_t *p8est_connectivity_new_shell (void);
  */
 p8est_connectivity_t *p8est_connectivity_new_sphere (void);
 
+/** Create a connectivity structure that builds a revolution torus.
+ *
+ * This connectivity reuses vertices and relies on a geometry transformation.
+ * It is thus not suitable for p8est_connectivity_complete.
+ *
+ * This connectivity reuses ideas from disk2d connectivity.
+ * More precisely the torus is divided into segments around
+ * the revolution axis, each segments is made of 5 trees (à la disk2d).
+ * The total number of trees if 5 times the number of segments.
+ *
+ * This connectivity is meant to be used with \ref p8est_geometry_new_torus
+ *
+ * \param[in] nSegments number of trees along the great circle
+ */
+p8est_connectivity_t *p8est_connectivity_new_torus (int nSegments);
+
 /** Create connectivity structure from predefined catalogue.
  * \param [in]  name            Invokes connectivity_new_* function.
  *              brick235        brick (2, 3, 5, 0, 0, 0)
@@ -589,13 +759,14 @@ p8est_connectivity_t *p8est_connectivity_new_byname (const char *name);
  * This is useful if you would like to uniformly refine by something other
  * than a power of 2.
  *
- * \param [in] conn         a valid connectivity
- * \param [in] num_per_edge the number of new trees in each direction
+ * \param [in] conn         A valid connectivity
+ * \param [in] num_per_dim  The number of new trees in each direction.
+ *                      Must use no more than \ref P8EST_OLD_QMAXLEVEL bits.
  *
  * \return a refined connectivity.
  */
 p8est_connectivity_t *p8est_connectivity_refine (p8est_connectivity_t * conn,
-                                                 int num_per_edge);
+                                                 int num_per_dim);
 
 /** Fill an array with the axis combination of a face neighbor transform.
  * \param [in]  iface       The number of the originating face.
@@ -619,12 +790,14 @@ void                p8est_expand_face_transform (int iface, int nface,
                                                  int ftransform[]);
 
 /** Fill an array with the axis combination of a face neighbor transform.
+ * \param [in]  connectivity Connectivity structure.
  * \param [in]  itree       The number of the originating tree.
  * \param [in]  iface       The number of the originating tree's face.
  * \param [out] ftransform  This array holds 9 integers.
  *              [0]..[2]    The coordinate axis sequence of the origin face.
  *              [3]..[5]    The coordinate axis sequence of the target face.
- *              [6]..[8]    Edge reverse flag for axes t1, t2; face code for n.
+ *              [6]..[8]    Edge reversal flag for axes t1, t2; face code for n;
+ *                          \see p8est_expand_face_transform.
  * \return                  The face neighbor tree if it exists, -1 otherwise.
  */
 p4est_topidx_t      p8est_find_face_transform (p8est_connectivity_t *
@@ -633,6 +806,7 @@ p4est_topidx_t      p8est_find_face_transform (p8est_connectivity_t *
                                                int iface, int ftransform[]);
 
 /** Fills an array with information about edge neighbors.
+ * \param [in] connectivity Connectivity structure.
  * \param [in] itree    The number of the originating tree.
  * \param [in] iedge    The number of the originating edge.
  * \param [in,out] ei   A p8est_edge_info_t structure with initialized array.
@@ -644,6 +818,7 @@ void                p8est_find_edge_transform (p8est_connectivity_t *
                                                p8est_edge_info_t * ei);
 
 /** Fills an array with information about corner neighbors.
+ * \param [in] connectivity  Connectivity structure.
  * \param [in] itree    The number of the originating tree.
  * \param [in] icorner  The number of the originating corner.
  * \param [in,out] ci   A p8est_corner_info_t structure with initialized array.
@@ -692,7 +867,7 @@ void                p8est_connectivity_permute (p8est_connectivity_t * conn,
 
 #ifdef P4EST_WITH_METIS
 
-/** p8est_connectivity_reorder
+/** Reorder a connectivity using METIS.
  * This function takes a connectivity \a conn and a parameter \a k,
  * which will typically be the number of processes, and reorders the trees
  * such that if every processes is assigned (num_trees / k) trees, the
@@ -714,10 +889,37 @@ void                p8est_connectivity_permute (p8est_connectivity_t * conn,
  * \param [in]     ctype      determines when an edge exists in the dual graph
  *                            of the connectivity structure.
  */
-void                p8est_connectivity_reorder (MPI_Comm comm, int k,
+void                p8est_connectivity_reorder (sc_MPI_Comm comm, int k,
                                                 p8est_connectivity_t * conn,
                                                 p8est_connect_type_t ctype);
 
+/** Reorder a connectivity using METIS.
+ * This is the same form of p8est_connectivity_reorder but it takes an initialized
+ * sc array \a newid as extra argument.
+ * In this way, the users can map old indices to new indices in the case it
+ * is necessary (for instance to retrieve high-order nodes previously stored
+ * in an array with old indices).
+ * \param [in]     comm       MPI communicator.
+ * \param [in]     k          if k > 0, the number of pieces metis will use to
+ *                            guide the reordering; if k = 0, the number of
+ *                            pieces will be determined from the MPI
+ *                            communicator.
+ * \param [in,out] conn       connectivity that will be reordered.
+ * \param [in]     ctype      determines when an edge exists in the dual graph
+ *                            of the connectivity structure.
+ * \param [in,out] newid      array that maps old tree indices to new ones.
+ *                            newid has to be an sc_array and it has to be
+ *                            initialized (non-NULL) with element size
+ *                            of size_t (using sc_array_new (sizeof (size_t))).
+ *                            Input length arbitrary, output length modified.
+ */
+sc_array_t         *p8est_connectivity_reorder_newid (sc_MPI_Comm comm, int k,
+                                                      p8est_connectivity_t *
+                                                      conn,
+                                                      p8est_connect_type_t
+                                                      ctype,
+                                                      sc_array_t * newid);
+
 #endif /* P4EST_WITH_METIS */
 
 /** p8est_connectivity_join_faces
